# -*- coding: iso-8859-15 -*-
"""This module tries to convert the unicode accented chars into latex strings.
Will be probably rewritten to use pylatexenc.

This file is part of the physbiblio package.
"""
import re
import traceback

from pylatexenc.latexencode import utf8tolatex

try:
    from physbiblio.errors import pBLogger
    from physbiblio.strings.main import ParseAccentsStrings as pastr
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())
    raise

accents_changed = []


def parse_accents_str(string):
    """Function that reads a string and translates
    all the known unicode characters into latex commands.

    Parameters:
        string: the string to be translated

    Output:
        the processed string
    """
    if string is not None and string != "":
        string = utf8tolatex(string, non_ascii_only=True)
    return string


def parse_accents_record(record):
    """Function that reads the fields inside a bibtex dictionary
    and translates all the known unicode characters into latex commands.

    Parameters:
        record: the bibtex dictionary generated by bibtexparser

    Output:
        the dictionary after having processed all the fields
    """
    for val in record:
        if val != "ID" and len(record[val].strip()) > 0:
            tmp = utf8tolatex(record[val], non_ascii_only=True)
            if tmp != record[val]:
                pBLogger.info(pastr.converting % record["ID"])
                pBLogger.info(pastr.infodashes + tmp.encode("utf-8"))
                accents_changed.append(record["ID"])
            record[val] = tmp
    return record


latex2Html_commands = [["textit", "i"], ["textbf", "b"]]
latex2Html_strings = [["\%", "%"], ["~", " "], ["\ ", " "]]
latex_replace = [["text", "rm"]]


def texToHtml(text):
    """Function that converts some Latex commands into HTML commands.

    Parameters:
        text: the string to be processed

    Output:
        the processed string
    """
    for tex, html in latex2Html_commands:
        match = re.compile("\\\\%s\{(.*| |\n)?\}" % tex, re.MULTILINE)
        for t in match.finditer(text):
            text = text.replace(
                t.group(), "<{html}>{cont}</{html}>".format(html=html, cont=t.group(1))
            )
    for tex, html in latex2Html_strings:
        text = text.replace(tex, html)
    for tex, new in latex_replace:
        match = re.compile("\\\\%s\{(.*| |\n)?\}" % tex, re.MULTILINE)
        for t in match.finditer(text):
            text = text.replace(t.group(), "\\%s{%s}" % (new, t.group(1)))
    return text



"""Utilities for in the tests of the physbiblio modules.

This file is part of the physbiblio package.
"""
import datetime
import os
import sys
import traceback
import unittest
from io import StringIO
from unittest.mock import patch

USE_AUTOSPEC_CLASS = True

today_ymd = datetime.datetime.today().strftime("%y%m%d_%H%M%S")
tempLogFileName = "tests_%s.log" % today_ymd

try:
    from physbiblio.config import pbConfig

    pbConfig.overWritelogFileName = os.path.join(pbConfig.dataPath, tempLogFileName)
    pbConfig.prepareLogger("physbibliotestlog")
    pbConfig.reloadProfiles()
    from physbiblio.database import PhysBiblioDB
    from physbiblio.errors import pBErrorManager, pBLogger
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())
    raise


class SkipTestsSettingsClass:
    """Store settings for deciding the tests to skip"""

    def __init__(self):
        """Define non-gui skip settings"""
        self.default()

    def copy(self):
        """create a new instance, copy of the previous one"""
        new = SkipTestsSettingsClass()
        new.db = bool(self.db)
        new.gui = bool(self.gui)
        new.long = bool(self.long)
        new.online = bool(self.online)
        return new

    def default(self):
        """Default settings"""
        self.db = False
        self.gui = False
        self.long = False
        self.online = False

    def __str__(self):
        """print current settings"""
        return "DB: %s\nGUI: %s\nlong: %s\nOnline: %s" % (
            self.db,
            self.gui,
            self.long,
            self.online,
        )


skipTestsSettings = SkipTestsSettingsClass()

pbConfig.params["logFileName"] = tempLogFileName
logFileName = os.path.join(pbConfig.dataPath, pbConfig.params["logFileName"])

tempDBName = os.path.join(pbConfig.dataPath, "tests_%s.db" % today_ymd)
if os.path.exists(tempDBName):
    os.remove(tempDBName)
tempFDBName = os.path.join(pbConfig.dataPath, "tests_first_%s.db" % today_ymd)
if os.path.exists(tempFDBName):
    os.remove(tempFDBName)


class NameSpace:
    pass


class DBTestCase(unittest.TestCase):
    """define the class that will be used for database tests"""

    @classmethod
    @patch("logging.Logger.debug")
    @patch("logging.Logger.info")
    @patch("logging.Logger.warning")
    @patch("logging.Logger.exception")
    def setUpClass(self, *args):
        """Create a temporary database"""
        self.maxDiff = None
        self.pBDB = PhysBiblioDB(tempDBName, pBLogger)

    def tearDown(self):
        """Clean temporary database"""
        self.pBDB.undo(verbose=False)

    @patch("sys.stdout", new_callable=StringIO)
    def assert_stdout(self, function, expected_output, mock_stdout):
        """Catch and test stdout of the function"""
        pBErrorManager.tempHandler(sys.stdout, format="%(message)s")
        function()
        pBErrorManager.rmTempHandler()
        self.assertEqual(mock_stdout.getvalue(), expected_output)


def tearDownModule():
    """Clean temporary files at the end"""
    if os.path.exists(logFileName):
        os.remove(logFileName)
    if os.path.exists(tempDBName):
        os.remove(tempDBName)
    if os.path.exists(tempFDBName):
        os.remove(tempFDBName)



"""Module that manages the command line calls of PhysBiblio.

This file is part of the physbiblio package.
"""
import argparse
import datetime
import sys

from PySide6.QtWidgets import QApplication

try:
    from physbiblio import __version__, __version_date__
    from physbiblio.config import pbConfig
    from physbiblio.errors import pBLogger
    from physbiblio.gui.errorManager import pBGUIErrorManager
    from physbiblio.strings.main import ArgParserStrings as apstr
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise


def call_citationCount(args):
    """Wrapper for citationCount + commit of the changes"""
    from physbiblio.database import pBDB

    inspireID = [e["inspire"] for e in pBDB.bibs.getAll()]
    pBDB.bibs.citationCount(inspireID)
    pBDB.commit()


def call_clean(args):
    """Function used when the "clean" subcommand is called"""
    from physbiblio.database import pBDB

    pBDB.bibs.cleanBibtexs(startFrom=args.startFrom)
    pBDB.commit()


def call_cli(args):
    """Function used when the "cli" subcommand is called"""
    from physbiblio.cli import cli as physBiblioCLI

    physBiblioCLI()


def call_export(args):
    """Function used when the "export" subcommand is called"""
    from physbiblio.export import pBExport

    pBExport.exportAll(args.filename)


def call_tests(args):
    """Function used when the "test" subcommand is called"""
    from physbiblio.setuptests import skipTestsSettings

    if args.database:
        skipTestsSettings.db = True
    if args.gui:
        skipTestsSettings.gui = True
    if args.long:
        skipTestsSettings.long = True
    if args.online:
        skipTestsSettings.online = True

    import unittest

    from physbiblio.testLoader import PBScanningLoader

    suite = PBScanningLoader().discover("physbiblio")
    testRunner = unittest.runner.TextTestRunner()
    result = testRunner.run(suite)
    if not result.wasSuccessful():
        raise Exception(apstr.testFailed)


def call_tex(args):
    """Function used when the "tex" subcommand is called"""
    from physbiblio.export import pBExport

    pBExport.exportForTexFile(
        args.texFiles,
        args.bibFile,
        overwrite=args.overwrite,
        autosave=args.save,
        updateExisting=args.updateExisting,
        removeUnused=args.removeUnused,
        reorder=args.reorder,
    )


def call_update(args):
    """Function used when the "update" subcommand is called"""
    from physbiblio.database import pBDB

    pBDB.bibs.searchOAIUpdates(startFrom=args.startFrom, force=args.force)
    pBDB.commit()


def cumulativeInspireDates(date1, date2):
    """Wrapper for getDailyInfoFromOAI + commit of the changes"""
    from physbiblio.database import pBDB

    pBDB.bibs.getDailyInfoFromOAI(date1, date2)
    pBDB.commit()


dateLast = datetime.date.today().strftime("%Y-%m-%d")


def call_dates(args):
    """Use cumulativeInspireDates to fetch updates between two dates
    specified in the command line
    """
    cumulativeInspireDates(args.start, args.end)


def call_daily(args):
    """Use cumulativeInspireDates to fetch daily updates"""
    cumulativeInspireDates(
        (datetime.date.today() - datetime.timedelta(1)).strftime("%Y-%m-%d"), dateLast
    )


def call_weekly(args):
    """Use cumulativeInspireDates to fetch weekly updates"""
    cumulativeInspireDates(
        (datetime.date.today() - datetime.timedelta(7)).strftime("%Y-%m-%d"), dateLast
    )


def call_gui(args=None):
    """Function that runs the PhysBiblio GUI"""
    # these two imports must stay here,
    # so they start after the profile has been loaded properly:
    try:
        import physbiblio.gui.mainWindow
        from physbiblio.database import pBDB
    except ImportError:
        print("Could not find physbiblio and its modules!")
        raise
    try:
        app = QApplication(sys.argv)
        mainWin = physbiblio.gui.mainWindow.MainWindow()
        sys.excepthook = mainWin.errormessage.emit
        mainWin.show()
        mainWin.raise_()
        if pbConfig.params["openSinceLastUpdate"] != __version__:
            mainWin.recentChanges()
            pbConfig.globalDb.config.update("openSinceLastUpdate", __version__)
            pbConfig.globalDb.commit()
        sys.exit(app.exec())
    except NameError:
        pBLogger.critical("NameError:", exc_info=True)
    except SystemExit:
        pBDB.closeDB()
        pBLogger.info(apstr.closeMainW)


class NewProfileAction(argparse.Action):
    """Used to trigger a reload of the settings
    if a profile is specified as an argument
    """

    def __init__(self, option_strings, dest, **kwargs):
        super(NewProfileAction, self).__init__(option_strings, dest, **kwargs)

    def __call__(self, parser, namespace, values, option_string=None):
        prof = values[0]
        if prof in pbConfig.profiles.keys():
            pbConfig.reloadProfiles(prof)
        setattr(namespace, self.dest, values)


def setParser():
    """Contains the argparse definition of sub-commands and options"""
    parser = argparse.ArgumentParser(prog="PhysBiblio.exe")
    parser.add_argument(
        "-p",
        "--profile",
        action=NewProfileAction,
        nargs=1,
        choices=pbConfig.profiles.keys(),
        help=apstr.profileHelp,
    )
    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version="PhysBiblio %s (%s)" % (__version__, __version_date__),
    )
    subparsers = parser.add_subparsers(help=apstr.subHelp, dest="cmd")

    parser_cit = subparsers.add_parser("citations", help=apstr.cleanHelp)
    parser_cit.set_defaults(func=call_citationCount)

    parser_clean = subparsers.add_parser("clean", help=apstr.cleanHelp)
    parser_clean.add_argument(
        "-s",
        "--startFrom",
        type=int,
        help=apstr.cleanStartHelp,
        default=0,
    )
    parser_clean.set_defaults(func=call_clean)

    parser_cli = subparsers.add_parser("cli", help=apstr.cliHelp)
    parser_cli.set_defaults(func=call_cli)

    parser_daily = subparsers.add_parser("daily", help=apstr.dailyHelp)
    parser_daily.set_defaults(func=call_daily)

    parser_dates = subparsers.add_parser("dates", help=apstr.datesHelp)
    parser_dates.add_argument(
        "start",
        metavar="startdate",
        help=apstr.datesStartHelp,
        default=(datetime.date.today() - datetime.timedelta(1)).strftime("%Y-%m-%d"),
    )
    parser_dates.add_argument(
        "end",
        metavar="enddate",
        help=apstr.datesEndHelp,
        default=dateLast,
    )
    parser_dates.set_defaults(func=call_dates)

    parser_export = subparsers.add_parser("export", help=apstr.exportHelp)
    parser_export.add_argument("filename", help=apstr.exportFilenameHelp)
    parser_export.set_defaults(func=call_export)

    parser_test = subparsers.add_parser("test", help=apstr.testHelp)
    parser_test.add_argument(
        "-d",
        "--database",
        dest="database",
        action="store_true",
        help=apstr.testDbHelp,
    )
    parser_test.add_argument(
        "-g", "--gui", dest="gui", action="store_true", help=apstr.testGuiHelp
    )
    parser_test.add_argument(
        "-l",
        "--long",
        dest="long",
        action="store_true",
        help=apstr.testLongHelp,
    )
    parser_test.add_argument(
        "-o",
        "--online",
        dest="online",
        action="store_true",
        help=apstr.testOnlineHelp,
    )
    parser_test.set_defaults(func=call_tests)

    parser_tex = subparsers.add_parser("tex", help=apstr.texHelp)
    parser_tex.add_argument(
        "bibFile",
        metavar="bibfilename",
        help=apstr.texBibHelp,
    )
    parser_tex.add_argument(
        "texFiles",
        metavar="texfilename",
        help=apstr.texTexsHelp,
        nargs="+",
    )
    parser_tex.add_argument(
        "-o",
        "--overwrite",
        dest="overwrite",
        action="store_true",
        help=apstr.texOverwriteHelp,
    )
    parser_tex.add_argument(
        "-r",
        "--removeUnused",
        dest="removeUnused",
        action="store_true",
        help=apstr.texRemoveHelp,
    )
    parser_tex.add_argument(
        "--reorder",
        dest="reorder",
        action="store_true",
        help=apstr.texReorderHelp,
    )
    parser_tex.add_argument(
        "-s",
        "--save",
        dest="save",
        action="store_false",
        help=apstr.texSaveHelp,
    )
    parser_tex.add_argument(
        "-u",
        "--updateExisting",
        dest="updateExisting",
        action="store_true",
        help=apstr.texUpdateHelp,
    )
    parser_tex.set_defaults(func=call_tex)

    parser_update = subparsers.add_parser("update", help=apstr.updateHelp)
    parser_update.add_argument(
        "-s",
        "--startFrom",
        type=int,
        help=apstr.updateStartHelp,
        default=0,
    )
    parser_update.add_argument(
        "-f", "--force", action="store_true", help=apstr.updateForceHelp
    )
    parser_update.set_defaults(func=call_update)

    parser_weekly = subparsers.add_parser("weekly", help=apstr.weeklyHelp)
    parser_weekly.set_defaults(func=call_weekly)

    parser_gui = subparsers.add_parser("gui", help=apstr.guiHelp)
    parser_gui.set_defaults(func=call_gui)

    return parser



#!/usr/bin/env python
"""Test file for the physbiblio.gui.errorManager module.

This file is part of the physbiblio package.
"""
import logging
import os
import traceback
import unittest
from unittest.mock import MagicMock, patch

from PySide6.QtCore import Qt
from PySide6.QtTest import QTest
from PySide6.QtWidgets import QMessageBox

try:
    from physbiblio.errors import PBErrorManagerClass
    from physbiblio.gui.errorManager import *
    from physbiblio.gui.setuptests import *
    from physbiblio.setuptests import *
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestGUIErrorManager(GUITestCase):
    """Test the functions in ErrorManager"""

    def test_ErrorStream(self):
        """Test functions in ErrorStream"""
        es = ErrorStream()
        self.assertIsInstance(es, StringIO)
        self.assertEqual(es.lastMBox, None)
        self.assertEqual(es.priority, 1)
        es.setPriority(2)
        self.assertEqual(es.priority, 2)
        mb = MagicMock()
        with patch("physbiblio.gui.errorManager.QMessageBox", return_value=mb) as _mb:
            es.write("sometext")
            _mb.assert_called_once_with(_mb.Information, "Information", "")
        mb.setText.assert_called_once_with("sometext")
        mb.setWindowTitle.assert_called_once_with("Error")
        mb.setIcon.assert_called_once_with(_mb.Critical)
        mb.exec.assert_called_once_with()
        self.assertEqual(es.priority, 1)
        es.setPriority(0)
        mb = MagicMock()
        with patch("physbiblio.gui.errorManager.QMessageBox", return_value=mb) as _mb:
            es.write("some\ntext")
            _mb.assert_called_once_with(_mb.Information, "Information", "")
        mb.setText.assert_called_once_with("some<br>text")
        mb.setWindowTitle.assert_called_once_with("Information")
        mb.setIcon.assert_called_once_with(_mb.Information)
        mb.exec.assert_called_once_with()
        mb = MagicMock()
        with patch("physbiblio.gui.errorManager.QMessageBox", return_value=mb) as _mb:
            es.write("some new text")
            _mb.assert_called_once_with(_mb.Information, "Information", "")
        mb.setText.assert_called_once_with("some new text")
        mb.setWindowTitle.assert_called_once_with("Warning")
        mb.setIcon.assert_called_once_with(_mb.Warning)
        mb.exec.assert_called_once_with()
        with patch("physbiblio.gui.errorManager.QMessageBox", return_value=mb) as _mb:
            self.assertEqual(es.write(" "), None)
            _mb.assert_not_called()

    def test_PBErrorManagerClassGui(self):
        """Test functions in ErrorStream"""
        el = PBErrorManagerClassGui()
        self.assertIsInstance(el, PBErrorManagerClass)
        self.assertIsInstance(el.guiStream, ErrorStream)
        l = el.loggerPriority(25)
        self.assertEqual(el.guiStream.priority, 25)
        self.assertIsInstance(l, logging.Logger)

    def test_objects(self):
        """test that the module contains the appropriate objects"""
        import physbiblio.gui.errorManager as pgem

        self.assertIsInstance(pgem.pBGUIErrorManager, pgem.PBErrorManagerClassGui)
        self.assertIsInstance(pgem.pBGUIErrorManager, PBErrorManagerClass)
        self.assertIsInstance(pgem.pBGUILogger, logging.Logger)
        self.assertEqual(pgem.pBGUILogger, pgem.pBGUIErrorManager.logger)


if __name__ == "__main__":
    unittest.main()



"""Module with the class that manages marks for the bibtex entries.

This file is part of the physbiblio package.
"""
import traceback

from PySide6.QtWidgets import QCheckBox, QGroupBox, QHBoxLayout, QRadioButton

try:
    import physbiblio.gui.resourcesPyside6
    from physbiblio.strings.gui import MarksStrings as mstr
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())


class Marks:
    """Class that manages the marks of bibtex entries"""

    def __init__(self):
        """Class constructor. Creates the 5 default marks."""
        self.marks = {}
        self.newMark("imp", mstr.important, "emblem-important-symbolic")
        self.newMark("fav", mstr.favorite, "emblem-favorite-symbolic")
        self.newMark("bad", mstr.bad, "emblem-remove")
        self.newMark("que", mstr.unclear, "emblem-question")
        self.newMark("new", mstr.toBeRead, "unread-new")

    def newMark(self, key, desc, icon):
        """Add a new mark.

        Parameters:
            key: the key to be used in the self.marks dictionary
            desc: the mark description
            icon: the name of the icon file (omitting the .png extension),
                which must be present in ":/images/"
        """
        self.marks[key] = {"desc": desc, "icon": ":/images/%s.png" % icon}

    def getGroupbox(self, marksData, description=mstr.marks, radio=False, addAny=False):
        """Create a `QGroupBox` containing `QCheckBox`s or `QRadioButton`s
        for marks selection

        Parameters:
            marksData: a list containing the keys of the elements
                that must be checked at the beginning
            description: the description/title of the `QGroupBox`
            radio (boolean, optional, default False):
                if True, use `QRadioButton` instead of `QCheckBox`
            addAny (boolean, optional, default False):
                add the "any" button to the `QGroupBox`

        Output:
            a list: (groupBox, markValues)
            groupBox: the `QGroupBox` object
            markValues: the list of `QCheckBox`s or `QRadioButton`s,
                for further values extraction
        """
        groupBox = QGroupBox(description)
        markValues = {}
        groupBox.setFlat(True)
        boxlayout = QHBoxLayout()
        for m, cont in self.marks.items():
            if radio:
                markValues[m] = QRadioButton(cont["desc"])
            else:
                markValues[m] = QCheckBox(cont["desc"])
            if marksData is not None and m in marksData:
                markValues[m].setChecked(True)
            boxlayout.addWidget(markValues[m])
        if addAny:
            markValues["any"] = QRadioButton(mstr.anys)
            boxlayout.addWidget(markValues["any"])
        groupBox.setLayout(boxlayout)
        return groupBox, markValues


pBMarks = Marks()



#!/usr/bin/env python
"""Test file for the physbiblio.gui.basicDialogs module.

This file is part of the physbiblio package.
"""
import os
import traceback
import unittest
from unittest.mock import MagicMock, call, patch

from PySide6.QtCore import Qt
from PySide6.QtTest import QTest
from PySide6.QtWidgets import QInputDialog, QWidget

try:
    from physbiblio.gui.basicDialogs import *
    from physbiblio.gui.setuptests import *
    from physbiblio.setuptests import *
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestDialogWindows(GUITestCase):
    """Test the functions in DialogWindows"""

    def test_askYesNo(self):
        """Test askYesNo"""
        mb = MagicMock()
        mb.addButton.side_effect = ["yes", "no", "yes", "no"]
        mb.clickedButton.side_effect = ["yes", "no"]
        with patch("physbiblio.gui.basicDialogs.QMessageBox", return_value=mb) as _mb:
            self.assertTrue(askYesNo("mymessage"))
            _mb.assert_called_once_with(_mb.Question, "Question", "mymessage")
            mb.addButton.assert_has_calls([call(_mb.Yes), call(_mb.No)])
            mb.setDefaultButton.assert_called_once_with("no")
            mb.exec.assert_called_once_with()
            mb.clickedButton.assert_called_once_with()
            _mb.reset_mock()
            self.assertFalse(askYesNo("mymessage", title="mytitle"))
            _mb.assert_called_once_with(_mb.Question, "mytitle", "mymessage")

    def test_infoMessage(self):
        """Test infoMessage"""
        mb = MagicMock()
        with patch("physbiblio.gui.basicDialogs.QMessageBox", return_value=mb) as _mb:
            infoMessage("mymessage")
            _mb.assert_called_once_with(_mb.Information, "Information", "mymessage")
            _mb.reset_mock()
            infoMessage("mymessage", title="mytitle")
            _mb.assert_called_once_with(_mb.Information, "mytitle", "mymessage")
            mb.exec.assert_called_once_with()

    def test_LongInfoMessage(self):
        """Test LongInfoMessage"""
        with patch(
            "physbiblio.gui.basicDialogs.LongInfoMessage.exec", autospec=True
        ) as _e:
            win = LongInfoMessage("mymessage")
            _e.assert_called_once_with()
        self.assertIsInstance(win, QDialog)
        self.assertEqual(win.windowTitle(), "Information")
        self.assertIsInstance(win.layout(), QGridLayout)
        self.assertEqual(win.layout(), win.gridlayout)
        self.assertIsInstance(win.layout().itemAtPosition(0, 0).widget(), QTextEdit)
        self.assertEqual(win.layout().itemAtPosition(0, 0).widget(), win.textarea)
        self.assertEqual(win.textarea.isReadOnly(), True)
        self.assertEqual(win.textarea.toPlainText(), "mymessage")
        self.assertIsInstance(win.layout().itemAtPosition(1, 1).widget(), QPushButton)
        self.assertEqual(win.layout().itemAtPosition(1, 1).widget(), win.okbutton)
        self.assertEqual(win.okbutton.text(), "OK")
        with patch("PySide6.QtWidgets.QDialog.close") as _c:
            QTest.mouseClick(win.okbutton, Qt.LeftButton)
            _c.assert_called_once_with(win)
        mb = MagicMock()
        with patch(
            "physbiblio.gui.basicDialogs.LongInfoMessage.exec", autospec=True
        ) as _e:
            win = LongInfoMessage("mymessage", "mytitle")
        self.assertEqual(win.windowTitle(), "mytitle")

    def test_askGenericText(self):
        """Test askGenericText"""
        p = MagicMock()
        q = QWidget()
        qid = MagicMock()
        qid.exec.side_effect = [True, False, True]
        qid.textValue.side_effect = ["abc", "def", "ghi"]
        with patch(
            "physbiblio.gui.basicDialogs.QInputDialog", return_value=qid
        ) as _qid:
            self.assertEqual(askGenericText("mymessage", "mytitle"), ("abc", True))
            qid.setInputMode.assert_called_once_with(_qid.TextInput)
            qid.setLabelText.assert_called_once_with("mymessage")
            qid.setTextValue.assert_called_once_with("")
            qid.setWindowTitle.assert_called_once_with("mytitle")
            qid.exec.assert_called_once_with()
            qid.textValue.assert_called_once_with()
            self.assertEqual(
                askGenericText("mymessage", "mytitle", parent=p), ("def", False)
            )
        with patch(
            "physbiblio.gui.basicDialogs.QInputDialog", return_value=qid
        ) as _qid:
            self.assertEqual(
                askGenericText("mymessage", "mytitle", q, "ABC"), ("ghi", True)
            )
            qid.setTextValue.assert_any_call("ABC")

    def test_askFileName(self):
        """Test askFileName"""
        fd = MagicMock()
        fd.exec.side_effect = [False, True, True]
        fd.selectedFiles.side_effect = [["abc", "def"], []]
        with patch("physbiblio.gui.basicDialogs.QFileDialog", return_value=fd) as _fd:
            self.assertEqual(
                askFileName(title="mytitle", dir="/tmp", filter="test: *.txt"),
                "",
            )
            _fd.assert_called_once_with(None, "mytitle", "/tmp", "test: *.txt")
            fd.setFileMode.assert_called_once_with(_fd.ExistingFile)
            fd.exec.assert_called_once_with()
            fd.selectedFiles.assert_not_called()
            _fd.reset_mock()
            self.assertEqual(askFileName(), "abc")
            _fd.assert_called_once_with(None, "Filename to use:", "", "")
            fd.selectedFiles.assert_called_once_with()
            self.assertEqual(askFileName(), "")

    def test_askFileNames(self):
        """Test askFileNames"""
        fd = MagicMock()
        fd.exec.side_effect = [False, True]
        fd.selectedFiles.return_value = ["abc", "def"]
        with patch("physbiblio.gui.basicDialogs.QFileDialog", return_value=fd) as _fd:
            self.assertEqual(
                askFileNames(title="mytitle", dir="/tmp", filter="test: *.txt"),
                [],
            )
            _fd.assert_called_once_with(None, "mytitle", "/tmp", "test: *.txt")
            fd.setFileMode.assert_called_once_with(_fd.ExistingFiles)
            fd.setOption.assert_called_once_with(_fd.DontConfirmOverwrite, True)
            fd.exec.assert_called_once_with()
            fd.selectedFiles.assert_not_called()
            _fd.reset_mock()
            self.assertEqual(askFileNames(), ["abc", "def"])
            _fd.assert_called_once_with(None, "Filename to use:", "", "")
            fd.selectedFiles.assert_called_once_with()

    def test_askSaveFileName(self):
        """Test askSaveFileName"""
        fd = MagicMock()
        fd.exec.side_effect = [False, True, True]
        fd.selectedFiles.side_effect = [["abc", "def"], []]
        with patch("physbiblio.gui.basicDialogs.QFileDialog", return_value=fd) as _fd:
            self.assertEqual(
                askSaveFileName(title="mytitle", dir="/tmp", filter="test: *.txt"),
                "",
            )
            _fd.assert_called_once_with(None, "mytitle", "/tmp", "test: *.txt")
            fd.setFileMode.assert_called_once_with(_fd.AnyFile)
            fd.setOption.assert_called_once_with(_fd.DontConfirmOverwrite, True)
            fd.exec.assert_called_once_with()
            fd.selectedFiles.assert_not_called()
            _fd.reset_mock()
            self.assertEqual(askSaveFileName(), "abc")
            _fd.assert_called_once_with(None, "Filename to use:", "", "")
            fd.selectedFiles.assert_called_once_with()
            self.assertEqual(askSaveFileName(), "")

    def test_askDirName(self):
        """Test askDirName"""
        fd = MagicMock()
        fd.exec.side_effect = [False, True, True]
        fd.selectedFiles.side_effect = [["abc", "def"], []]
        with patch("physbiblio.gui.basicDialogs.QFileDialog", return_value=fd) as _fd:
            self.assertEqual(askDirName(title="mytitle", dir="/tmp"), "")
            _fd.assert_called_once_with(None, "mytitle", "/tmp")
            fd.setFileMode.assert_called_once_with(_fd.Directory)
            fd.setOption.assert_called_once_with(_fd.ShowDirsOnly, True)
            fd.exec.assert_called_once_with()
            fd.selectedFiles.assert_not_called()
            _fd.reset_mock()
            self.assertEqual(askDirName(), "abc")
            _fd.assert_called_once_with(None, "Directory to use:", "")
            fd.selectedFiles.assert_called_once_with()
            self.assertEqual(askDirName(), "")


if __name__ == "__main__":
    unittest.main()



"""Static definitions of the structure (column names and content)
for the tables of the database.

This file is part of the physbiblio package.
"""
import traceback

try:
    from physbiblio.strings.main import TablesDefStrings as tdstr
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())

profilesSettingsTable = [
    ["name", "text", "primary key not null"],
    ["databasefile", "text", "not null"],
    ["description", "text", ""],
    ["oldCfg", "text", ""],
    ["isDefault", "integer", "default 0"],
    ["ord", "integer", "default 100"],
    ["CONSTRAINT", "unique_databasefile", "UNIQUE (databasefile)"],
]
searchesTable = [
    ["idS", "integer", "primary key"],
    ["name", "text", ""],
    ["count", "integer", "default 0 not null"],
    ["searchDict", "text", "not null"],
    ["limitNum", "integer", "default 50"],
    ["offsetNum", "integer", "default 0"],
    ["replaceFields", "text", ""],
    ["manual", "integer", "default 0"],
    ["isReplace", "integer", "default 0"],
]
searchesTableDescriptions = tdstr.searchesDescs

tableFields = {}
tableFields["entries"] = [
    ["bibkey", "text", "primary key not null collate nocase"],
    ["inspire", "text", ""],
    ["arxiv", "text", ""],
    ["ads", "text", ""],
    ["scholar", "text", ""],
    ["doi", "text", ""],
    ["isbn", "text", ""],
    ["year", "integer", ""],
    ["link", "text", ""],
    ["comments", "text", ""],
    ["old_keys", "text", ""],
    ["crossref", "text", ""],
    ["bibtex", "text", "not null"],
    ["firstdate", "text", "not null"],
    ["pubdate", "text", ""],
    ["exp_paper", "integer", "default 0"],
    ["lecture", "integer", "default 0"],
    ["phd_thesis", "integer", "default 0"],
    ["review", "integer", "default 0"],
    ["proceeding", "integer", "default 0"],
    ["book", "integer", "default 0"],
    ["noUpdate", "integer", "default 0"],
    ["marks", "text", ""],
    ["abstract", "text", ""],
    ["bibdict", "text", ""],
    ["citations", "integer", "default 0"],
    ["citations_no_self", "integer", "default 0"],
]
tableFields["categories"] = [
    ["idCat", "integer", "primary key"],
    ["name", "text", "not null"],
    ["description", "text", "not null"],
    ["parentCat", "integer", "default 0"],
    ["comments", "text", "default ''"],
    ["ord", "integer", "default 0"],
]
tableFields["experiments"] = [
    ["idExp", "integer", "primary key"],
    ["name", "text", "not null"],
    ["comments", "text", "not null"],
    ["homepage", "text", ""],
    ["inspire", "text", ""],
]
tableFields["expCats"] = [
    ["idExC", "integer", "primary key"],
    ["idExp", "integer", "not null"],
    ["idCat", "integer", "not null"],
]
tableFields["entryCats"] = [
    ["idEnC", "integer", "primary key"],
    ["bibkey", "text", "not null"],
    ["idCat", "integer", "not null"],
]
tableFields["entryExps"] = [
    ["idEnEx", "integer", "primary key"],
    ["bibkey", "text", "not null"],
    ["idExp", "integer", "not null"],
]
tableFields["settings"] = [
    ["name", "text", "primary key not null"],
    ["value", "text", "default ''"],
]
fieldsDescriptions = {}
fieldsDescriptions["entries"] = tdstr.entriesDescs
fieldsDescriptions["categories"] = tdstr.catsDescs
fieldsDescriptions["experiments"] = tdstr.expsDescs
fieldsDescriptions["expCats"] = tdstr.expsCatsDescs
fieldsDescriptions["entryCats"] = tdstr.entriesCatsDescs
fieldsDescriptions["entryExps"] = tdstr.entriesExpsDescs
fieldsDescriptions["settings"] = tdstr.settingsDescs



from .common import CommonStrings


class GenericStrings(CommonStrings):
    """Generic strings that are used in the entire module"""

    genericError = "Impossible to get results"
    searchInfo = "Search '%s' -> %s"


class ADSNasaStrings:
    """Strings for the adsnasa module"""

    genericExportError = "Something went wrong while exporting ADS query"
    genericFetchError = "Something went wrong while fetching ADS"
    invalidField = "Invalid requested field in ADS query: %s"
    remainingQueries = "Remaining queries: %s/%s"
    unauthorized = "Unauthorized use of ADS API. Is your token valid?"


class ArxivStrings(GenericStrings):
    """Strings for the arxiv module"""

    cannotFindEntriesInFeed = "Key 'entries' not found in processed feed:\n%s"
    cannotParseRSS = "Cannot parse arxiv RSS feed:\n%s"
    cannotReadDict = "Something went wrong when reading the dictionary"
    cannotReadEntryId = "Cannot read the arXiv ID of this entry: %s"
    cannotReadItem = "Something went wrong when reading the feed item"
    emptyUrl = "Url is empty!"
    errorYearConversion = "Error in converting year from '%s'"
    mainCatNotFound = "Main category not found: %s"
    searchInfo = "Search '%s:%s' -> %s"
    subCatNotFound = "Sub category not found: %s"


class DOIStrings(GenericStrings):
    """Strings for the doi module"""


class InspireStrings(GenericStrings):
    """Strings for the inspire module"""

    apiResponseError = (
        "An error occurred. The API returned the following message: %s - %s"
    )
    endString = "END --- %s \n\n"
    errorEmptyText = "An error occurred. Empty text obtained"
    errorEmptySearch = "An error occurred. No search results obtained"
    errorInvalidBibtex = "Invalid bibtex!\n%s"
    errorReadRecord = "Error in readRecord! id:%s"
    errorUpdateBibtex = "Cannot update bibtex!"
    exceptionFormat = "%s, %s\n%s"
    foundID = "Found: %s"
    jsonError = "Cannot load JSON content"
    numberOfEntriesChanged = "The number of records changed from %s to %s during the execution of the INSPIRE API query"
    processed = "Processed %d elements"
    readData = "Reading data --- "
    searchIDInfo = "Search ID of %s -> %s"
    searchResultsFrom = "Search info from %s"
    startString = "\nSTARTING INSPIRE API harvester --- %s \n\n"
    warningJournal = "'journal' from OAI is missing or not a string (recid:%s)"
    warningMissingField = "Field '%s' from OAI is missing (recid:%s)"


class WebInterfStrings:
    """Strings for the webInterf module"""

    errorBadCodification = "[%s] -> Bad codification, utf-8 decode failed"
    errorImportMethod = "Error importing physbiblio.webimport.%s"
    errorNotFound = "[%s] -> %s not found"
    errorRetrieve = "[%s] -> Error in retrieving data from url"
    errorTimedOut = "[%s] -> Timed out"
    methodNotAvailable = "The method '%s' is not available!"



#!/usr/bin/env python
"""Test file for the physbiblio.gui.marks module.

This file is part of the physbiblio package.
"""
import os
import traceback

from PySide6.QtCore import Qt
from PySide6.QtTest import QTest
from PySide6.QtWidgets import QHBoxLayout

try:
    from physbiblio.gui.marks import *
    from physbiblio.gui.setuptests import *
    from physbiblio.setuptests import *
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestMarks(GUITestCase):
    """Test the functions in marks.Marks"""

    def test_pbmarks(self):
        """test that pBMarks exists"""
        self.assertIsInstance(pBMarks, Marks)

    def testInit(self):
        """tests for the marks.__init__ function"""
        m = Marks()
        self.assertEqual(len(m.marks.keys()), 5)
        for q in ("imp", "fav", "bad", "que", "new"):
            self.assertIn(q, m.marks.keys())
        self.assertEqual(
            m.marks["imp"],
            {"desc": "Important", "icon": ":/images/emblem-important-symbolic.png"},
        )
        self.assertEqual(
            m.marks["fav"],
            {"desc": "Favorite", "icon": ":/images/emblem-favorite-symbolic.png"},
        )
        self.assertEqual(
            m.marks["bad"], {"desc": "Bad", "icon": ":/images/emblem-remove.png"}
        )
        self.assertEqual(
            m.marks["que"], {"desc": "Unclear", "icon": ":/images/emblem-question.png"}
        )
        self.assertEqual(
            m.marks["new"], {"desc": "To be read", "icon": ":/images/unread-new.png"}
        )

    def testNewMark(self):
        """tests for the marks.newMark function"""
        m = Marks()
        m.newMark("abc", "desc", "icon")
        self.assertEqual(m.marks["abc"], {"desc": "desc", "icon": ":/images/icon.png"})

    def testGetGroupbox(self):
        """tests for the marks.getGroupbox function"""
        m = Marks()
        gb, mv = m.getGroupbox(None)
        self.assertTrue(gb.isFlat())
        self.assertEqual(gb.title(), "Marks")
        self.assertIsInstance(gb.layout(), QHBoxLayout)
        self.assertIsInstance(mv, dict)
        for q in ("imp", "fav", "bad", "que", "new"):
            self.assertIn(q, mv.keys())
        self.assertNotIn("any", mv.keys())
        for qcb in mv.values():
            self.assertIsInstance(qcb, QCheckBox)
            self.assertFalse(qcb.isChecked())

        gb, mv = m.getGroupbox(["new", "imp"])
        for q, qcb in mv.items():
            self.assertIsInstance(mv[q], QCheckBox)
            if q in ("new", "imp"):
                self.assertTrue(qcb.isChecked())
            else:
                self.assertFalse(qcb.isChecked())

        gb, mv = m.getGroupbox(["imp"], "try", radio=True)
        self.assertEqual(gb.title(), "try")
        for q, qcb in mv.items():
            self.assertIsInstance(mv[q], QRadioButton)
            if q in ("imp"):
                self.assertTrue(qcb.isChecked())
            else:
                self.assertFalse(qcb.isChecked())

        gb, mv = m.getGroupbox([], radio=True, addAny=True)
        self.assertIn("any", mv.keys())
        for q, qcb in mv.items():
            self.assertIsInstance(mv[q], QRadioButton)
            self.assertFalse(qcb.isChecked())


if __name__ == "__main__":
    unittest.main()



"""Utilities for in the tests of the physbiblio modules.

This file is part of the physbiblio package.
"""
import traceback
import unittest
from io import StringIO
from unittest.mock import MagicMock, patch

from PySide6.QtWidgets import QApplication, QFileDialog

try:
    from physbiblio.gui.mainWindow import MainWindow
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())

globalQApp = QApplication()
globalQApp.exec = MagicMock(return_value=0)


def fakeExec(x, string, out):
    """Simulate the selection of some files and
    return True/False as if confirmed/canceled
    """
    QFileDialog.selectFile(x, string)
    return out


class GUITestCase(unittest.TestCase):
    """Class that manages GUI tests"""

    @classmethod
    def setUpClass(self):
        """Assign a temporary QApplication to the class instance"""
        self.maxDiff = None
        self.qapp = globalQApp

    @classmethod
    def tearDownClass(self):
        """Remove a temporary QApplication from the class instance"""
        del self.qapp

    def assertGeometry(self, obj, x, y, w, h):
        """Test the geometry of an object

        Parameters:
            obj: the object to be tested
            x: the expected x position
            y: the expected y position
            w: the expected width
            h: the expected height
        """
        geom = obj.geometry()
        self.assertEqual(geom.x(), x)
        self.assertEqual(geom.y(), y)
        self.assertEqual(geom.width(), w)
        self.assertEqual(geom.height(), h)


class GUIwMainWTestCase(unittest.TestCase):
    """Class that manages GUI tests which need a testing instance
    of the MainWindow class
    """

    @classmethod
    def setUpClass(self):
        """Call the parent method and instantiate a testing MainWindow"""
        super(GUIwMainWTestCase, self).setUpClass()
        self.mainW = MainWindow(testing=True)



"""The package that contains all the modules used by the PhysBiblio,
including the `gui` subpackage.

This file is part of the physbiblio package.
"""
__author__ = "Stefano Gariazzo"
__email__ = "stefano.gariazzo@gmail.com"

__all__ = [
    "bibtexWriter",
    "cli",
    "config",
    "databaseCore",
    "database",
    "errors",
    "export",
    "inspireStats",
    "parseAccents",
    "pdf",
    "strings",
    "tablesDef",
    "view",
    "tests",
    "gui",
    "webimport",
]

from .version import __recent_changes__, __version__, __version_date__



#!/usr/bin/env python
"""Test file for the physbiblio.gui.threadElements module.

This file is part of the physbiblio package.
"""
import os
import traceback
import unittest
from queue import Queue
from unittest.mock import MagicMock, call, patch
from urllib.request import URLError

from PySide6.QtCore import Qt, Signal
from PySide6.QtTest import QTest
from PySide6.QtWidgets import QWidget

try:
    from physbiblio import __version__
    from physbiblio.database import pBDB
    from physbiblio.gui.setuptests import *
    from physbiblio.gui.threadElements import *
    from physbiblio.inspireStats import pBStats
    from physbiblio.setuptests import *
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_checkUpdated(GUITestCase):
    """Test the functions in threadElements.Thread_checkUpdated"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        thr = Thread_checkUpdated(p)
        self.assertIsInstance(thr, PBThread)
        self.assertIsInstance(thr.result, Signal)
        self.assertEqual(thr.parent(), p)
        self.assertRaises(NotImplementedError, thr.setStopFlag)

    def test_run(self):
        """test run"""

        def fake_urlerror(x, y):
            raise URLError("no connection")

        p = QWidget()
        h1 = MagicMock()
        thr = Thread_checkUpdated(p)
        thr.result.connect(h1)
        with patch(
            "physbiblio.gui.threadElements.check_outdated",
            return_value=(False, __version__),
            autospec=True,
        ) as _cho:
            thr.run()
            _cho.assert_called_once_with("physbiblio", __version__)
        h1.assert_called_once_with(False, __version__)
        h1.reset_mock()
        with patch(
            "physbiblio.gui.threadElements.check_outdated",
            return_value=(),
            autospec=True,
        ) as _cho, patch("logging.Logger.warning") as _w:
            thr.run()
            _cho.assert_called_once_with("physbiblio", __version__)
            _w.assert_called_once_with(
                "Error when executing check_outdated. "
                + "Maybe you are using a developing version",
                exc_info=True,
            )
        h1.assert_not_called()
        with patch(
            "physbiblio.gui.threadElements.check_outdated", new=fake_urlerror
        ) as _cho, patch("logging.Logger.warning") as _w:
            thr.run()
            _w.assert_called_once_with(
                "Error when trying to check new versions. Are you offline?",
                exc_info=True,
            )
        with patch(
            "physbiblio.gui.threadElements.check_outdated",
            side_effect=ConnectionError,
            autospec=True,
        ) as _cho, patch("logging.Logger.warning") as _w:
            thr.run()
            _w.assert_called_once_with(
                "Error when trying to check new versions. Are you offline?",
                exc_info=True,
            )


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_citationCount(GUITestCase):
    """Test the functions in threadElements.Thread_citationCount"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_citationCount(ws, 123, p, pbMax="m", pbVal="v")
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.inspireID, 123)
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.pbMax, "m")
        self.assertEqual(thr.pbVal, "v")

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_citationCount(ws, 123, p, pbMax="m", pbVal="v")
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.database.Entries.citationCount", autospec=True
        ) as _cc, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _cc.assert_called_once_with(
                pBDB.bibs,
                123,
                pbMax="m",
                pbVal="v",
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)

    def test_setStopFlag(self):
        """test setStopFlag"""
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_citationCount(ws, 0)
        pBDB.bibs.runningCitationCount = True
        self.assertTrue(pBDB.bibs.runningCitationCount)
        thr.setStopFlag()
        self.assertFalse(pBDB.bibs.runningCitationCount)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_updateAllBibtexs(GUITestCase):
    """Test the functions in threadElements.Thread_updateAllBibtexs"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_updateAllBibtexs(ws, 123, p, [[]], True, "r", pbMax="m", pbVal="v")
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.startFrom, 123)
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.useEntries, [[]])
        self.assertEqual(thr.force, True)
        self.assertEqual(thr.reloadAll, "r")
        self.assertEqual(thr.pbMax, "m")
        self.assertEqual(thr.pbVal, "v")

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_updateAllBibtexs(
            ws, 123, p, [[]], True, True, pbMax="m", pbVal="v"
        )
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.database.Entries.searchOAIUpdates", autospec=True
        ) as _sou, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _sou.assert_called_once_with(
                pBDB.bibs,
                123,
                entries=[[]],
                force=True,
                reloadAll=True,
                pbMax="m",
                pbVal="v",
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)

    def test_setStopFlag(self):
        """test setStopFlag"""
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_updateAllBibtexs(ws, 0)
        pBDB.bibs.runningOAIUpdates = True
        self.assertTrue(pBDB.bibs.runningOAIUpdates)
        thr.setStopFlag()
        self.assertFalse(pBDB.bibs.runningOAIUpdates)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_replace(GUITestCase):
    """Test the functions in threadElements.Thread_replace"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_replace(
            ws, "a", ["b"], "1", ["2"], parent=p, regex=True, pbMax="m", pbVal="v"
        )
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.fiOld, "a")
        self.assertEqual(thr.fiNew, ["b"])
        self.assertEqual(thr.old, "1")
        self.assertEqual(thr.new, ["2"])
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.regex, True)
        self.assertEqual(thr.pbMax, "m")
        self.assertEqual(thr.pbVal, "v")

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_replace(ws, "a", ["b"], "1", ["2"], p, True, pbMax="m", pbVal="v")
        self.assertTrue(ws.running)
        pBDB.bibs.lastFetched = ["e", "f"]
        with patch(
            "physbiblio.database.Entries.replace",
            return_value=("r1", "r2", "r3"),
            autospec=True,
        ) as _rep, patch("time.sleep") as _sl, patch(
            "physbiblio.database.Entries.fetchFromLast",
            return_value=pBDB.bibs,
            autospec=True,
        ) as _ffl, patch(
            "physbiblio.database.Entries.fetchCursor",
            return_value="curs",
            autospec=True,
        ) as _fc, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _rep.assert_called_once_with(
                pBDB.bibs,
                "a",
                ["b"],
                "1",
                ["2"],
                entries="curs",
                lenEntries=2,
                regex=True,
                pbMax="m",
                pbVal="v",
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)
            _ffl.assert_has_calls([call(pBDB.bibs, doFetch=False), call(pBDB.bibs)])
            _fc.assert_called_once_with(pBDB.bibs)
            self.assertEqual(p.replaceResults, ("r1", "r2", "r3"))

    def test_setStopFlag(self):
        """test setStopFlag"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_replace(ws, "a", ["b"], "1", ["2"], p)
        pBDB.bibs.runningReplace = True
        self.assertTrue(pBDB.bibs.runningReplace)
        thr.setStopFlag()
        self.assertFalse(pBDB.bibs.runningReplace)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_updateInspireInfo(GUITestCase):
    """Test the functions in threadElements.Thread_updateInspireInfo"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_updateInspireInfo(ws, "Gariazzo:2015rra", 1385583, p)
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.bibkey, "Gariazzo:2015rra")
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.inspireID, 1385583)
        self.assertRaises(NotImplementedError, thr.setStopFlag)

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_updateInspireInfo(ws, "Gariazzo:2015rra", 1385583, p)
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.database.Entries.updateInfoFromOAI", autospec=True
        ) as _fun, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(
                pBDB.bibs, 1385583, originalKey="Gariazzo:2015rra", verbose=1
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)
        for iid in (None, False, ""):
            thr = Thread_updateInspireInfo(ws, "Gariazzo:2015rra", iid, p)
            with patch(
                "physbiblio.database.Entries.updateInfoFromOAI", autospec=True
            ) as _fun, patch(
                "physbiblio.database.Entries.updateInspireID",
                return_value=1385,
                autospec=True,
            ) as _uiid, patch(
                "time.sleep"
            ) as _sl, patch(
                "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
            ) as _st:
                thr.run()
                _uiid.assert_called_once_with(pBDB.bibs, "Gariazzo:2015rra")
                _fun.assert_called_once_with(
                    pBDB.bibs, 1385, originalKey=None, verbose=1
                )
                self.assertFalse(ws.running)
                _st.assert_called_once_with(ws)
                _sl.assert_called_once_with(0.1)

        ws = WriteStream(q)
        thr = Thread_updateInspireInfo(
            ws, ["Gariazzo:2015rra", "Gariazzo:2018mwd"], [1385583, None], p
        )
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.database.Entries.updateInfoFromOAI", autospec=True
        ) as _fun, patch(
            "physbiblio.database.Entries.updateInspireID",
            return_value=1385,
            autospec=True,
        ) as _uiid, patch(
            "time.sleep"
        ) as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_has_calls(
                [
                    call(pBDB.bibs, 1385583, originalKey="Gariazzo:2015rra", verbose=1),
                    call(pBDB.bibs, 1385, originalKey=None, verbose=1),
                ]
            )
            _uiid.assert_called_once_with(pBDB.bibs, "Gariazzo:2018mwd")
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_downloadArxiv(GUITestCase):
    """Test the functions in threadElements.Thread_downloadArxiv"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        thr = Thread_downloadArxiv("Gariazzo:2015rra", p)
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.bibkey, "Gariazzo:2015rra")
        self.assertRaises(NotImplementedError, thr.setStopFlag)

    def test_run(self):
        """test run"""
        p = QWidget()
        thr = Thread_downloadArxiv("Gariazzo:2015rra", p)
        with patch("physbiblio.pdf.LocalPDF.downloadArxiv", autospec=True) as _fun:
            thr.run()
            _fun.assert_called_once_with(pBPDF, "Gariazzo:2015rra")


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_processLatex(GUITestCase):
    """Test the functions in threadElements.Thread_processLatex"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        func = MagicMock()
        thr = Thread_processLatex(func, p)
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.func, func)
        self.assertIsInstance(thr.passData, Signal)
        self.assertRaises(NotImplementedError, thr.setStopFlag)

    def test_run(self):
        """test run"""
        p = QWidget()
        func = MagicMock(return_value=(["a", "b"], "text"))
        thr = Thread_processLatex(func, p)
        with MagicMock() as h1:
            thr.passData.connect(h1)
            thr.run()
            func.assert_called_once_with()
            h1.assert_called_once_with(["a", "b"], "text")


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_authorStats(GUITestCase):
    """Test the functions in threadElements.Thread_authorStats"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_authorStats(ws, "Stefano.Gariazzo.1", p, pbMax="m", pbVal="v")
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.authorName, "Stefano.Gariazzo.1")
        self.assertEqual(thr.receiver, ws)
        self.assertFalse(p.lastAuthorStats)
        self.assertEqual(thr.pbMax, "m")
        self.assertEqual(thr.pbVal, "v")
        self.assertRaises(
            Exception, lambda: Thread_authorStats(ws, "Stefano.Gariazzo.1", None)
        )

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_authorStats(ws, "Stefano.Gariazzo.1", p, pbMax="m", pbVal="v")
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.inspireStats.InspireStatsLoader.authorStats",
            return_value={"a": "b"},
            autospec=True,
        ) as _fun, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(
                pBStats, "Stefano.Gariazzo.1", pbMax="m", pbVal="v"
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)
            self.assertEqual(p.lastAuthorStats, {"a": "b"})

    def test_setStopFlag(self):
        """test setStopFlag"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_authorStats(ws, "Stefano.Gariazzo.1", p)
        pBStats.runningAuthorStats = True
        self.assertTrue(pBStats.runningAuthorStats)
        thr.setStopFlag()
        self.assertFalse(pBStats.runningAuthorStats)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_paperStats(GUITestCase):
    """Test the functions in threadElements.Thread_paperStats"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_paperStats(ws, 1385583, p, pbMax="m", pbVal="v")
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.inspireId, 1385583)
        self.assertEqual(thr.receiver, ws)
        self.assertFalse(p.lastPaperStats)
        self.assertEqual(thr.pbMax, "m")
        self.assertEqual(thr.pbVal, "v")
        self.assertRaises(Exception, lambda: Thread_paperStats(ws, 1385583, None))

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_paperStats(ws, 1385583, p, pbMax="m", pbVal="v")
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.inspireStats.InspireStatsLoader.paperStats",
            return_value={"a": "b"},
            autospec=True,
        ) as _fun, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(pBStats, 1385583, pbMax="m", pbVal="v")
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)
            self.assertEqual(p.lastPaperStats, {"a": "b"})

    def test_setStopFlag(self):
        """test setStopFlag"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_paperStats(ws, [1385583], p)
        pBStats.runningAuthorStats = True
        self.assertTrue(pBStats.runningPaperStats)
        thr.setStopFlag()
        self.assertFalse(pBStats.runningPaperStats)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_loadAndInsert(GUITestCase):
    """Test the functions in threadElements.Thread_loadAndInsert"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_loadAndInsert(ws, "Gariazzo:2015rra", p, pbMax="m", pbVal="v")
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.content, "Gariazzo:2015rra")
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.pbMax, "m")
        self.assertEqual(thr.pbVal, "v")
        self.assertRaises(Exception, lambda: Thread_loadAndInsert(ws, 1385583, None))

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_loadAndInsert(ws, "Gariazzo:2015rra", p, pbMax="m", pbVal="v")
        self.assertTrue(ws.running)
        pBDB.bibs.lastInserted = ["def"]
        with patch(
            "physbiblio.database.Entries.loadAndInsert",
            side_effect=[True, False],
            autospec=True,
        ) as _fun, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(
                pBDB.bibs, "Gariazzo:2015rra", pbMax="m", pbVal="v"
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)
            self.assertEqual(p.loadedAndInserted, ["def"])
            thr.run()
            self.assertEqual(p.loadedAndInserted, [])

    def test_setStopFlag(self):
        """test setStopFlag"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_loadAndInsert(ws, "Gariazzo:2015rra", p)
        pBDB.bibs.runningLoadAndInsert = True
        self.assertTrue(pBDB.bibs.runningLoadAndInsert)
        thr.setStopFlag()
        self.assertFalse(pBDB.bibs.runningLoadAndInsert)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_cleanAllBibtexs(GUITestCase):
    """Test the functions in threadElements.Thread_cleanAllBibtexs"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_cleanAllBibtexs(ws, 123, p, [[]], pbMax="m", pbVal="v")
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.startFrom, 123)
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.useEntries, [[]])
        self.assertEqual(thr.pbMax, "m")
        self.assertEqual(thr.pbVal, "v")

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_cleanAllBibtexs(ws, 123, p, [[]], pbMax="m", pbVal="v")
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.database.Entries.cleanBibtexs", autospec=True
        ) as _fun, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(
                pBDB.bibs, 123, entries=[[]], pbMax="m", pbVal="v"
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)

    def test_setStopFlag(self):
        """test setStopFlag"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_cleanAllBibtexs(ws, 123, p, [[]])
        pBDB.bibs.runningCleanBibtexs = True
        self.assertTrue(pBDB.bibs.runningCleanBibtexs)
        thr.setStopFlag()
        self.assertFalse(pBDB.bibs.runningCleanBibtexs)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_findBadBibtexs(GUITestCase):
    """Test the functions in threadElements.Thread_findBadBibtexs"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_findBadBibtexs(ws, 123, p, [[]], pbMax="m", pbVal="v")
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.startFrom, 123)
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.useEntries, [[]])
        self.assertEqual(thr.pbMax, "m")
        self.assertEqual(thr.pbVal, "v")
        self.assertRaises(Exception, lambda: Thread_loadAndInsert(ws, 1385583, None))

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_findBadBibtexs(ws, 123, p, [[]], pbMax="m", pbVal="v")
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.database.Entries.findCorruptedBibtexs", autospec=True
        ) as _fun, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(
                pBDB.bibs, 123, entries=[[]], pbMax="m", pbVal="v"
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)

    def test_setStopFlag(self):
        """test setStopFlag"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_findBadBibtexs(ws, 123, p, [[]])
        pBDB.bibs.runningFindBadBibtexs = True
        self.assertTrue(pBDB.bibs.runningFindBadBibtexs)
        thr.setStopFlag()
        self.assertFalse(pBDB.bibs.runningFindBadBibtexs)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_importFromBib(GUITestCase):
    """Test the functions in threadElements.Thread_importFromBib"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_importFromBib(ws, "tmp.bib", False, p, pbMax="m", pbVal="v")
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.bibFile, "tmp.bib")
        self.assertFalse(thr.complete)
        self.assertEqual(thr.pbMax, "m")
        self.assertEqual(thr.pbVal, "v")

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_importFromBib(ws, "tmp.bib", False, p, pbMax="m", pbVal="v")
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.database.Entries.importFromBib", autospec=True
        ) as _fun, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(
                pBDB.bibs, "tmp.bib", False, pbMax="m", pbVal="v"
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)

    def test_setStopFlag(self):
        """test setStopFlag"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_importFromBib(ws, "tmp.bib", False, p)
        pBDB.bibs.importFromBibFlag = True
        self.assertTrue(pBDB.bibs.importFromBibFlag)
        thr.setStopFlag()
        self.assertFalse(pBDB.bibs.importFromBibFlag)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_exportTexBib(GUITestCase):
    """Test the functions in threadElements.Thread_exportTexBib"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_exportTexBib(ws, ["main.tex"], "biblio.bib", p)
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.outFName, "biblio.bib")
        self.assertEqual(thr.texFiles, ["main.tex"])
        self.assertFalse(thr.updateExisting)
        self.assertFalse(thr.removeUnused)
        self.assertFalse(thr.reorder)
        thr = Thread_exportTexBib(
            ws,
            ["main.tex"],
            "biblio.bib",
            removeUnused=True,
            updateExisting=True,
            reorder=True,
            parent=p,
        )
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.outFName, "biblio.bib")
        self.assertEqual(thr.texFiles, ["main.tex"])
        self.assertTrue(thr.updateExisting)
        self.assertTrue(thr.removeUnused)
        self.assertTrue(thr.reorder)

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_exportTexBib(ws, ["main.tex"], "biblio.bib", p)
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.export.PBExport.exportForTexFile", autospec=True
        ) as _fun, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(
                pBExport,
                ["main.tex"],
                "biblio.bib",
                removeUnused=False,
                updateExisting=False,
                reorder=False,
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)
        ws = WriteStream(q)
        thr = Thread_exportTexBib(
            ws,
            ["main.tex"],
            "biblio.bib",
            p,
            updateExisting=True,
            removeUnused=True,
            reorder=True,
        )
        thr = Thread_exportTexBib(ws, ["main.tex"], "biblio.bib", p, True, True, True)
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.export.PBExport.exportForTexFile", autospec=True
        ) as _fun, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(
                pBExport,
                ["main.tex"],
                "biblio.bib",
                removeUnused=True,
                updateExisting=True,
                reorder=True,
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)

    def test_setStopFlag(self):
        """test setStopFlag"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_exportTexBib(ws, ["main.tex"], "biblio.bib", p)
        pBExport.exportForTexFlag = True
        self.assertTrue(pBExport.exportForTexFlag)
        thr.setStopFlag()
        self.assertFalse(pBExport.exportForTexFlag)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_cleanSpare(GUITestCase):
    """Test the functions in threadElements.Thread_cleanSpare"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_cleanSpare(ws, p)
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.receiver, ws)
        self.assertRaises(NotImplementedError, thr.setStopFlag)

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_cleanSpare(ws, p)
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.database.Utilities.cleanSpareEntries", autospec=True
        ) as _fun, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(pBDB.utils)
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_cleanSparePDF(GUITestCase):
    """Test the functions in threadElements.Thread_cleanSparePDF"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_cleanSparePDF(ws, p)
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.receiver, ws)
        self.assertRaises(NotImplementedError, thr.setStopFlag)

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_cleanSparePDF(ws, p)
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.pdf.LocalPDF.removeSparePDFFolders", autospec=True
        ) as _fun, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(pBPDF)
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_fieldsArxiv(GUITestCase):
    """Test the functions in threadElements.Thread_fieldsArxiv"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_fieldsArxiv(
            ws, ["a", "b"], ["abstract", "arxiv"], p, pbMax="m", pbVal="v"
        )
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.entries, ["a", "b"])
        self.assertEqual(thr.fields, ["abstract", "arxiv"])
        self.assertEqual(thr.pbMax, "m")
        self.assertEqual(thr.pbVal, "v")

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_fieldsArxiv(
            ws, ["a", "b"], ["abstract", "arxiv"], p, pbMax="m", pbVal="v"
        )
        self.assertTrue(ws.running)
        with patch(
            "physbiblio.database.Entries.getFieldsFromArxiv", autospec=True
        ) as _fun, patch("time.sleep") as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            _fun.assert_called_once_with(
                pBDB.bibs, ["a", "b"], ["abstract", "arxiv"], pbMax="m", pbVal="v"
            )
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)

    def test_setStopFlag(self):
        """test setStopFlag"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_fieldsArxiv(ws, ["a", "b"], ["abstract", "arxiv"], p)
        pBDB.bibs.getArxivFieldsFlag = True
        self.assertTrue(pBDB.bibs.getArxivFieldsFlag)
        thr.setStopFlag()
        self.assertFalse(pBDB.bibs.getArxivFieldsFlag)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class Test_Thread_importDailyArxiv(GUITestCase):
    """Test the functions in threadElements.Thread_importDailyArxiv"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_importDailyArxiv(ws, ["a", "b"], p)
        self.assertIsInstance(thr, PBThread)
        self.assertEqual(thr.parent(), p)
        self.assertEqual(thr.receiver, ws)
        self.assertEqual(thr.found, ["a", "b"])
        self.assertTrue(thr.runningImport)

    def test_run(self):
        """test run"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        self.assertTrue(ws.running)
        thr = Thread_importDailyArxiv(
            ws,
            {
                "12.345": {
                    "bibpars": {
                        "author": "me",
                        "title": "title",
                        "type": "",
                        "eprint": "12.345",
                        "replacement": False,
                        "cross": False,
                        "abstract": "some text",
                        "primaryclass": "astro-ph",
                    },
                    "exist": 1,
                }
            },
            p,
        )
        with patch(
            "physbiblio.database.Entries.loadAndInsert",
            return_value=True,
            autospec=True,
        ) as _lai, patch(
            "physbiblio.database.Entries.getByKey",
            return_value=[{"bibkey": "12.345"}],
            autospec=True,
        ) as _gbk, patch(
            "time.sleep"
        ) as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st, patch(
            "logging.Logger.info"
        ) as _in:
            thr.run()
            _lai.assert_called_once_with(pBDB.bibs, "12.345")
            _gbk.assert_called_once_with(pBDB.bibs, "12.345")
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)
            _in.assert_has_calls(
                [
                    call("Entries successfully imported: ['12.345']"),
                    call("Errors for entries:\n[]"),
                ]
            )

        q = Queue()
        ws = WriteStream(q)
        self.assertTrue(ws.running)
        thr = Thread_importDailyArxiv(
            ws,
            {
                "12.345": {
                    "bibpars": {
                        "author": "me1",
                        "title": "title1",
                        "type": "",
                        "eprint": "12.345",
                        "replacement": False,
                        "cross": False,
                        "abstract": "some text",
                        "primaryclass": "astro-ph",
                    },
                    "exist": 1,
                },
                "12.346": {
                    "bibpars": {
                        "author": "me2",
                        "title": "title2",
                        "type": "",
                        "eprint": "12.346",
                        "replacement": False,
                        "cross": True,
                        "abstract": "some other text",
                        "primaryclass": "astro-ph.CO",
                    },
                    "exist": 1,
                },
                "12.348": {
                    "bibpars": {
                        "author": "me4",
                        "title": "title4",
                        "type": "",
                        "eprint": "12.348",
                        "replacement": False,
                        "cross": False,
                        "abstract": "some more text",
                        "primaryclass": "hep-ph",
                    },
                    "exist": 1,
                },
                "12.349": {
                    "bibpars": {
                        "author": "me5",
                        "title": "title5",
                        "type": "",
                        "eprint": "12.349",
                        "replacement": False,
                        "cross": False,
                        "abstract": "some more text",
                        "primaryclass": "hep-ex",
                    },
                    "exist": 1,
                },
            },
            p,
        )
        with patch(
            "physbiblio.database.Entries.loadAndInsert",
            side_effect=[True, False, False, False],
            autospec=True,
        ) as _lai, patch(
            "physbiblio.database.Entries.prepareInsert",
            side_effect=["data1", "data2", "data3"],
            autospec=True,
        ) as _pi, patch(
            "physbiblio.database.Entries.updateInspireID",
            return_value="123",
            autospec=True,
        ) as _uid, patch(
            "physbiblio.database.Entries.searchOAIUpdates", autospec=True
        ) as _sou, patch(
            "physbiblio.database.Entries.insert",
            side_effect=[False, True, True],
            autospec=True,
        ) as _bi, patch(
            "physbiblio.database.Entries.getByKey",
            side_effect=[[{"bibkey": "12.345"}], [], [{"bibkey": "12.350"}]],
            autospec=True,
        ) as _gbk, patch(
            "physbiblio.database.Entries.getByBibkey",
            return_value=[{"bibkey": "12.346"}],
            autospec=True,
        ) as _gbb, patch(
            "logging.Logger.info"
        ) as _in, patch(
            "logging.Logger.warning"
        ) as _wa, patch(
            "time.sleep"
        ) as _sl, patch(
            "physbiblio.gui.commonClasses.WriteStream.start", autospec=True
        ) as _st:
            thr.run()
            self.assertFalse(ws.running)
            _st.assert_called_once_with(ws)
            _sl.assert_called_once_with(0.1)
            _lai.assert_has_calls(
                [
                    call(pBDB.bibs, "12.345"),
                    call(pBDB.bibs, "12.346"),
                    call(pBDB.bibs, "12.348"),
                    call(pBDB.bibs, "12.349"),
                ]
            )
            _gbk.assert_has_calls(
                [
                    call(pBDB.bibs, "12.345"),
                    call(pBDB.bibs, "12.348"),
                    call(pBDB.bibs, "12.349"),
                ]
            )
            _in.assert_has_calls(
                [
                    call("Element '12.348' successfully inserted.\n"),
                    call("Element '12.349' successfully inserted.\n"),
                    call(
                        "Entries successfully imported: "
                        + "['12.345', '12.348', '12.350']"
                    ),
                    call("Errors for entries:\n['12.346', '12.348']"),
                ]
            )
            _wa.assert_has_calls(
                [
                    call("Failed in inserting entry '12.346'\n"),
                    call(
                        "Failed in completing info for entry '12.348'\n", exc_info=True
                    ),
                ]
            )
            _pi.assert_has_calls(
                [
                    call(
                        pBDB.bibs,
                        '@Article{12.346,\n        author = "me2",\n'
                        + '         title = "{title2}",\n archiveprefix '
                        + '= "arXiv",\n  primaryclass = "astro-ph.CO",\n'
                        + '        eprint = "12.346",\n}\n\n',
                    ),
                    call(
                        pBDB.bibs,
                        '@Article{12.348,\n        author = "me4",\n'
                        + '         title = "{title4}",\n archiveprefix '
                        + '= "arXiv",\n  primaryclass = "hep-ph",\n'
                        + '        eprint = "12.348",\n}\n\n',
                    ),
                    call(
                        pBDB.bibs,
                        '@Article{12.349,\n        author = "me5",\n'
                        + '         title = "{title5}",\n archiveprefix '
                        + '= "arXiv",\n  primaryclass = "hep-ex",\n'
                        + '        eprint = "12.349",\n}\n\n',
                    ),
                ]
            )
            _bi.assert_has_calls([call(pBDB.bibs, "data1"), call(pBDB.bibs, "data2")])
            _uid.assert_has_calls(
                [call(pBDB.bibs, "12.348"), call(pBDB.bibs, "12.349")]
            )
            _sou.assert_has_calls(
                [
                    call(
                        pBDB.bibs,
                        0,
                        entries=[{"bibkey": "12.346"}],
                        force=True,
                        reloadAll=True,
                    ),
                    call(
                        pBDB.bibs,
                        0,
                        entries=[{"bibkey": "12.346"}],
                        force=True,
                        reloadAll=True,
                    ),
                ]
            )
            _gbb.assert_has_calls(
                [call(pBDB.bibs, "12.348"), call(pBDB.bibs, "12.349")]
            )

    def test_setStopFlag(self):
        """test setStopFlag"""
        p = QWidget()
        q = Queue()
        ws = WriteStream(q)
        thr = Thread_importDailyArxiv(ws, ["a", "b"], p)
        self.assertTrue(thr.runningImport)
        thr.setStopFlag()
        self.assertFalse(thr.runningImport)


if __name__ == "__main__":
    unittest.main()



"""Module that uses INSPIRE-HEP to get author
(number of papers, h index, citations) and paper statistics (citations).

Uses matplotlib to do plots.

This file is part of the physbiblio package.
"""
import datetime
import json
import os
import os.path as osp
import traceback

import dateutil
import matplotlib
import matplotlib.dates as mdates
import matplotlib.pyplot as plt
import pytz
from matplotlib.backends.backend_pdf import PdfPages

plt.switch_backend("QtAgg")
os.environ["QT_API"] = "pyside6"

try:
    from physbiblio.config import pbConfig
    from physbiblio.errors import pBLogger
    from physbiblio.strings.main import InspireStatsStrings as isstr
    from physbiblio.webimport.webInterf import physBiblioWeb
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())
    raise


class InspireStatsLoader:
    """Class that contains the methods
    to collect information from INSPIRE-HEP
    """

    authorPlotInfo = {}
    paperPlotInfo = {}

    def __init__(self):
        """The class constructor,
        defines some constants and search options
        """
        self.authorPlotInfo = None
        self.paperPlotInfo = None
        self.allInfoA = {}
        self.authorPapersList = [[], []]
        self.allCitations = []
        self.runningAuthorStats = True
        self.runningPaperStats = True
        self.allInfoP = {}
        self.citingPapersList = [[], []]

    def changeBackend(self, wantBackend):
        """Changes the matplotlib backend currently in use.

        Parameters:
            wantBackend: a string that defines the wanted backend
        """
        if wantBackend != matplotlib.get_backend():
            matplotlib.use(wantBackend, warn=False, force=True)
            from matplotlib import pyplot as plt

            pBLogger.info(isstr.changeBackend % matplotlib.get_backend())

    def authorStats(self, authorName, plot=False, reset=True, pbMax=None, pbVal=None):
        """Function that gets the data and
        constructs the statistics for a given author.

        Parameters:
            authorName: the author name as identified into INSPIRE-HEP,
                or a list of author names
                (it calls itself recursively for all the list elements)
            plot (boolean, default False): True to call self.plotStats
            reset (boolean, default False): True to delete
                all previous existing data
                (used as False when processing a list of authors)
            pbMax (callable, optional): a function to set the maximum
                of a progress bar in the GUI, if possible
            pbVal (callable, optional): a function to set the value
                of a progress bar in the GUI, if possible

        Output:
            a dictionary containing all the statistic information.
            For a single author, the structure is the following:
            {
                "name": the author name,
                "aI": The complete information,
                    including the dictionaries with the single papers info
                    (see self.paperStats), the citations
                    and the corresponding dates,
                "paLi": a list of [id, date] of the papers
                    associated with the author,
                "allLi": the complete list of [date, total citations]
                    with all the citations to the papers,
                "meanLi": the complete list of
                    [date, total citations/number of papers]
                    computed at each point from "allLi" content,
                "h": the h-index,
                "figs" (only if `plot` is True): contains the figures.
                    See self.plotStats
            }
        """
        if reset:
            self.allInfoA = {}
            self.authorPapersList = [[], []]
            self.allCitations = []
        if isinstance(authorName, list):
            try:
                pbMax(len(authorName))
            except TypeError:
                pass
            for ia, a in enumerate(authorName):
                try:
                    pbVal(ia + 1)
                except TypeError:
                    pass
                self.authorStats(a, reset=False)
            self.authorPlotInfo["name"] = authorName
            return self.authorPlotInfo
        pBLogger.info(isstr.authorStats % authorName)
        data, tot = physBiblioWeb.webSearch["inspire"].retrieveSearchResults(
            "author:%s" % authorName, fields=["control_number"]
        )
        recid_authorPapers = sorted(["%s" % a["id"] for a in data])
        tot = len(recid_authorPapers)
        pBLogger.info(isstr.authorStatsProcess % tot)
        self.runningAuthorStats = True
        try:
            pbMax(tot)
        except TypeError:
            pass
        batchSize = pbConfig.params["batchSizeInspire"]
        for i in range(0, tot, batchSize):
            entries, nume = physBiblioWeb.webSearch["inspire"].retrieveBatchQuery(
                recid_authorPapers[i : i + batchSize],
                searchFormat="recid:%s",
                fields=["control_number", "references.record"],
            )
            references, numr = physBiblioWeb.webSearch["inspire"].retrieveBatchQuery(
                recid_authorPapers[i : i + batchSize],
                searchFormat="refersto:recid:%s",
                fields=["control_number", "references.record"],
            )
            # fill here list of citations for each entry, use references->reference:record
            current = []
            for e in entries:
                eid = e["id"]
                refs = []
                for r in references:
                    if eid in [
                        a["record"]["$ref"].replace(
                            physBiblioWeb.webSearch["inspire"].url, ""
                        )
                        for a in r["metadata"]["references"]
                    ]:
                        refs.append(r)
                current.append([e, refs])

            for ix, (e, r) in enumerate(current):
                try:
                    pbVal(i + ix + 1)
                except TypeError:
                    pass
                if not self.runningAuthorStats:
                    pBLogger.info(isstr.stopReceived)
                    break
                p = e["id"]
                if p in self.allInfoA.keys():
                    continue
                self.allInfoA[p] = {}
                self.allInfoA[p]["date"] = dateutil.parser.parse(e["created"])
                self.authorPapersList[0].append(self.allInfoA[p]["date"])
                pBLogger.info(
                    isstr.authorStatsLooking
                    % (i + ix + 1, tot, 100.0 * (i + ix + 1) / tot, p)
                )
                paperInfo = self.paperStats(
                    p, verbose=0, paperDate=self.allInfoA[p]["date"], useData=r
                )
                self.allInfoA[p]["infoDict"] = paperInfo["aI"]
                self.allInfoA[p]["citingPapersList"] = paperInfo["citList"]
                for c, v in self.allInfoA[p]["infoDict"].items():
                    self.allCitations.append(v["date"])
                pBLogger.info("")

        self.authorPapersList[1] = []
        for i, p in enumerate(sorted(self.authorPapersList[0])):
            self.authorPapersList[0][i] = p
            self.authorPapersList[1].append(i + 1)
        pBLogger.info(isstr.savingCitations)
        allCitList = [[], []]
        meanCitList = [[], []]
        currPaper = 0
        for i, d in enumerate(sorted(self.allCitations)):
            if (
                currPaper < len(self.authorPapersList[0]) - 1
                and d >= self.authorPapersList[0][currPaper + 1]
            ):
                currPaper += 1
            allCitList[0].append(d)
            allCitList[1].append(i + 1)
            meanCitList[0].append(d)
            meanCitList[1].append((i + 1.0) / self.authorPapersList[1][currPaper])
        hind = 0
        citations = [
            len(self.allInfoA[k]["citingPapersList"][0]) - 2
            for k in self.allInfoA.keys()
        ]
        for h in range(len(citations)):
            if len([a for a in citations if a >= h]) >= h:
                hind = h
        self.authorPlotInfo = {
            "name": authorName,
            "aI": self.allInfoA,
            "paLi": self.authorPapersList,
            "allLi": allCitList,
            "meanLi": meanCitList,
            "h": hind,
        }
        if plot:
            self.authorPlotInfo["figs"] = self.plotStats(author=True)
        pBLogger.info(isstr.authorStatsCompleted % authorName)
        return self.authorPlotInfo

    def paperStats(
        self,
        paperID,
        plot=False,
        verbose=1,
        paperDate=None,
        reset=True,
        pbMax=None,
        pbVal=None,
        useData=None,
    ):
        """Function that gets the data and
        constructs the statistics for a given paper.

        Parameters:
            paperID (string): the INSPIRE-HEP id of the paper (a number)
            plot (boolean): whether or not the citations
                should be plotted (default False)
            verbose (int, default 1): increase the verbosity level
            paperDate (datetime, optional): the date of at which
                the paper was published
            reset (boolean, default False): True to delete
                all previous existing data
                (used as False when processing a list of IDs)
            pbMax (callable, optional): a function to set the maximum
                of a progress bar in the GUI, if possible
            pbVal (callable, optional): a function to set the value
                of a progress bar in the GUI, if possible
            useData (default None): if not None, the data to use
                instead of reading them from INSPIRE

        Output:
            a dictionary containing all the desired information.
            The structure is the following:
            {
                "id": the paper ID,
                "aI": the list of creation date for all the papers,
                    in INSPIRE-HEP order,
                "citList": the ordered list of citing papers,
                "fig" (only if `plot` is True): contains the figure.
                    See self.plotStats
            }
        """
        if reset:
            self.allInfoP = {}
            self.citingPapersList = [[], []]
        if isinstance(paperID, list):
            self.runningPaperStats = True
            try:
                pbMax(len(paperID))
            except TypeError:
                pass
            for ia, a in enumerate(paperID):
                try:
                    pbVal(ia + 1)
                except TypeError:
                    pass
                if self.runningPaperStats:
                    self.paperStats(a, reset=False)
            self.paperPlotInfo["id"] = paperID
            return self.paperPlotInfo
        if verbose > 0:
            pBLogger.info(isstr.paperStats % paperID)
        if useData is not None:
            data = useData
        else:
            data, tot = physBiblioWeb.webSearch["inspire"].retrieveSearchResults(
                "refersto:recid:%s" % paperID, fields=["control_number"]
            )
        self.readPaperStats(paperID, data, paperDate=paperDate)
        if plot:
            self.paperPlotInfo["fig"] = self.plotStats(paper=True)
        if verbose > 0:
            pBLogger.info(isstr.doneE)
        return self.paperPlotInfo

    def plotStats(
        self,
        paper=False,
        author=False,
        show=False,
        save=False,
        path=".",
        markPapers=False,
        pickVal=6,
    ):
        """Plot the collected information, using matplotlib.pyplot.

        Parameters:
            paper (boolean, default False): plot statistics
                for the last analyzed paper
            author (boolean, default False): plot statistics
                for the last analyzed author
            show (boolean, default False): True to show the plots
                in a separate window (with matplotlib.pyplot.show())
            save (boolean, default False): True to save the plots into files.
            path (string): where to save the plots
            markPapers (boolean, default False): True to draw
                a vertical lines at the dates
                corresponding to a paper appearing
            pickVal (float, default 6): the picker tolerance

        Output:
            False if paper==False and author==False,
            the matplotlib.pyplot figure containing
                the citation plot if paper==True,
            a list of matplotlib.pyplot figures containing
                the various plots if author==True
        """
        if paper and self.paperPlotInfo is not None:
            if len(self.paperPlotInfo["citList"][0]) > 0:
                pBLogger.info(isstr.plotPaper % self.paperPlotInfo["id"])
                fig, ax = plt.subplots()
                plt.plot(
                    self.paperPlotInfo["citList"][0],
                    self.paperPlotInfo["citList"][1],
                    picker=True,
                    pickradius=pickVal,
                )
                fig.autofmt_xdate()
                if save:
                    pdf = PdfPages(osp.join(path, self.paperPlotInfo["id"] + ".pdf"))
                    pdf.savefig()
                    pdf.close()
                if show:
                    plt.show()
                plt.close()
                return fig
        elif author and self.authorPlotInfo is not None:
            pBLogger.info(isstr.plotAuthor % self.authorPlotInfo["name"])
            try:
                ymin = min(
                    int(self.authorPlotInfo["allLi"][0][0].strftime("%Y")) - 2,
                    int(self.authorPlotInfo["paLi"][0][0].strftime("%Y")) - 2,
                )
                ymax = max(
                    int(self.authorPlotInfo["allLi"][0][-1].strftime("%Y")) + 2,
                    int(self.authorPlotInfo["paLi"][0][-1].strftime("%Y")) + 2,
                )
            except:
                try:
                    ymin = int(self.authorPlotInfo["paLi"][0][0].strftime("%Y")) - 2
                    ymax = int(self.authorPlotInfo["paLi"][0][-1].strftime("%Y")) + 2
                except:
                    pBLogger.warning(isstr.noPublications)
                    return False
            figs = []
            # total number of papers
            if len(self.authorPlotInfo["paLi"][0]) > 0:
                fig, ax = plt.subplots()
                plt.title(isstr.paperNumber)
                plt.plot(
                    self.authorPlotInfo["paLi"][0],
                    self.authorPlotInfo["paLi"][1],
                    picker=True,
                    pickradius=pickVal,
                )
                fig.autofmt_xdate()
                if save:
                    pdf = PdfPages(
                        osp.join(path, self.authorPlotInfo["name"] + "_papers.pdf")
                    )
                    pdf.savefig()
                    pdf.close()
                if show:
                    plt.show()
                plt.close()
                figs.append(fig)

            # number of papers in each year
            if len(self.authorPlotInfo["paLi"][0]) > 0:
                fig, ax = plt.subplots()
                plt.title(isstr.paperYear)
                ax.hist(
                    [int(q.strftime("%Y")) for q in self.authorPlotInfo["paLi"][0]],
                    bins=range(ymin, ymax),
                    picker=True,
                )
                ax.get_xaxis().get_major_formatter().set_useOffset(False)
                plt.xlim([ymin, ymax])
                if save:
                    pdf = PdfPages(
                        osp.join(path, self.authorPlotInfo["name"] + "_yearPapers.pdf")
                    )
                    pdf.savefig()
                    pdf.close()
                if show:
                    plt.show()
                plt.close()
                figs.append(fig)

            # total number of citations
            if len(self.authorPlotInfo["allLi"][0]) > 0:
                fig, ax = plt.subplots()
                plt.title(isstr.totalCitations)
                plt.plot(
                    self.authorPlotInfo["allLi"][0],
                    self.authorPlotInfo["allLi"][1],
                    picker=True,
                    pickradius=pickVal,
                )
                fig.autofmt_xdate()
                if save:
                    pdf = PdfPages(
                        osp.join(path, self.authorPlotInfo["name"] + "_allCit.pdf")
                    )
                    pdf.savefig()
                    pdf.close()
                if show:
                    plt.show()
                plt.close()
                figs.append(fig)

            # number of citations in each year
            if len(self.authorPlotInfo["allLi"][0]) > 0:
                fig, ax = plt.subplots()
                plt.title(isstr.citationsYear)
                ax.hist(
                    [int(q.strftime("%Y")) for q in self.authorPlotInfo["allLi"][0]],
                    bins=range(ymin, ymax),
                    picker=True,
                )
                ax.get_xaxis().get_major_formatter().set_useOffset(False)
                plt.xlim([ymin, ymax])
                if save:
                    pdf = PdfPages(
                        osp.join(path, self.authorPlotInfo["name"] + "_yearCit.pdf")
                    )
                    pdf.savefig()
                    pdf.close()
                if show:
                    plt.show()
                plt.close()
                figs.append(fig)

            # mean number of citations per paper
            if len(self.authorPlotInfo["meanLi"][0]) > 0:
                fig, ax = plt.subplots()
                plt.title(isstr.meanCitations)
                plt.plot(
                    self.authorPlotInfo["meanLi"][0],
                    self.authorPlotInfo["meanLi"][1],
                    picker=True,
                    pickradius=pickVal,
                )
                fig.autofmt_xdate()
                if markPapers:
                    for q in self.authorPlotInfo["paLi"][0]:
                        plt.axvline(
                            datetime.datetime(
                                int(q.strftime("%Y")),
                                int(q.strftime("%m")),
                                int(q.strftime("%d")),
                            ),
                            color="k",
                            ls="--",
                        )
                if save:
                    pdf = PdfPages(
                        osp.join(path, self.authorPlotInfo["name"] + "_meanCit.pdf")
                    )
                    pdf.savefig()
                    pdf.close()
                if show:
                    plt.show()
                plt.close()
                figs.append(fig)

            # number of citations per paper
            if len(self.authorPlotInfo["aI"].keys()) > 0:
                fig, ax = plt.subplots()
                plt.title(isstr.citationsPaper)
                for i, p in enumerate(self.authorPlotInfo["aI"].keys()):
                    try:
                        plt.plot(
                            self.authorPlotInfo["aI"][p]["citingPapersList"][0],
                            self.authorPlotInfo["aI"][p]["citingPapersList"][1],
                        )
                    except:
                        pBLogger.exception(isstr.errorPlotting)
                fig.autofmt_xdate()
                if save:
                    pdf = PdfPages(
                        osp.join(path, self.authorPlotInfo["name"] + "_paperCit.pdf")
                    )
                    pdf.savefig()
                    pdf.close()
                if show:
                    plt.show()
                plt.close()
                figs.append(fig)
            return figs
        else:
            pBLogger.info(isstr.noPlot)
            return False

    def readPaperStats(
        self,
        paperID,
        data,
        paperDate=None,
    ):
        """Function that reads the data on a paper and
        constructs the corresponding statistics.

        Parameters:
            paperID (string): the INSPIRE-HEP id of the paper (a number)
            data (string): the INSPIRE-HEP id of the paper (a number)
            paperDate (datetime, optional): the date of at which
                the paper was published

        Output:
            a dictionary containing all the desired information.
            The structure is the following:
            {
                "id": the paper ID,
                "aI": the list of creation date for all the papers,
                    in INSPIRE-HEP order,
                "citList": the ordered list of citing papers,
                "fig" (only if `plot` is True): contains the figure.
                    See self.plotStats
            }
        """
        recid_citingPapers = [a["id"] for a in data]
        if paperDate is not None:
            self.citingPapersList[0].append(paperDate)
        for i, p in enumerate(recid_citingPapers):
            self.allInfoP[p] = {}
            self.allInfoP[p]["date"] = dateutil.parser.parse(data[i]["created"])
            self.citingPapersList[0].append(
                self.allInfoP[p]["date"].replace(tzinfo=pytz.UTC)
            )
        for i, p in enumerate(sorted(self.citingPapersList[0])):
            self.citingPapersList[0][i] = p
            self.citingPapersList[1].append(i + 1)
        self.citingPapersList[0].append(
            datetime.datetime.fromordinal(datetime.date.today().toordinal()).replace(
                tzinfo=pytz.UTC
            )
        )
        try:
            self.citingPapersList[1].append(self.citingPapersList[1][-1])
        except IndexError:
            self.citingPapersList[1].append(0)
        self.paperPlotInfo = {
            "id": paperID,
            "aI": self.allInfoP,
            "citList": self.citingPapersList,
        }


pBStats = InspireStatsLoader()



"""Module containing strings used while communicating with the user
in the physbiblio package.

This module is part of the physbiblio package.
"""
from physbiblio import __author__, __email__, __version__, __version_date__

__all__ = [
    "common",
    "gui",
    "main",
    "webimport",
]



"""Module that creates the base class WebInterf,
which will be used by other modules in this package.

Uses urllib to download url content.

This file is part of the physbiblio package.
"""
import os
import pkgutil
import socket
import ssl
import traceback
from urllib.request import HTTPError, Request, URLError, urlopen

import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

try:
    import physbiblio.webimport as wi
    from physbiblio.config import pbConfig
    from physbiblio.errors import pBLogger
    from physbiblio.strings.webimport import WebInterfStrings
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())
    raise

# scan package content to load list of available modules, to be imported later
pkgpath = os.path.dirname(wi.__file__)
webInterfaces = [name for _, name, _ in pkgutil.iter_modules([pkgpath])]


class PBSession(requests.Session):
    """Extends the Session class to include auto-retries"""

    def __init__(
        self,
        total_retries=5,
        backoff=1.0,
        status_forcelist=[429, 500, 502, 503, 504],
        method_whitelist=["HEAD", "GET", "OPTIONS"],
        **kwargs
    ):
        """Extend the Session class.
        Input parameters are as from
        requests.packages.urllib3.util.retry.Retry or requests.Session.
        """
        retry_strategy = Retry(
            total=total_retries,
            backoff_factor=backoff,
            status_forcelist=status_forcelist,
            allowed_methods=method_whitelist,
        )
        adapter = HTTPAdapter(max_retries=retry_strategy)
        requests.Session.__init__(self, **kwargs)
        self.mount("https://", adapter)
        self.mount("http://", adapter)


class WebInterf(WebInterfStrings):
    """This is the main class for the web search methods.

    It contains a constructor, a function to create an appropriate url
    and to retrieve text from the url,
    a function to load other webinterfaces
    """

    url = None
    urlArgs = None
    urlTimeout = 1000.0
    interfaces = []

    def __init__(self):
        """Initializes the class variables."""
        self.urlTimeout = float(pbConfig.params["timeoutWebSearch"])
        # save the names of the available web search interfaces
        self.interfaces = [
            a for a in webInterfaces if a not in ("strings", "tests", "webInterf")
        ]
        self.webSearch = {}
        self.loaded = False

    def createUrl(self, args=None, url=None):
        """Joins the arguments of the GET query to get the full url.

        Uses the self.urlArgs dictionary to generate the list
        of HTTP GET parameters, unless args is passed as an argument.

        Parameter:
            args (default None): a dictionary
            url (default None): a string

        Output:
            a string
        """
        if not isinstance(args, dict):
            args = self.urlArgs
        if not isinstance(url, str) or url == "":
            url = self.url
        return (
            url + "?" + "&".join(["%s=%s" % (a, b) for a, b in args.items()])
            if len(args) > 0
            else url
        )

    def textFromUrl(self, url, headers=None):
        """Use urllib to get the html content of the given url.

        Parameters:
            url: the url to be opened
            headers (default None): the additional headers
                to be passed to urllib.Request

        Output:
            text: the content of the url
        """
        http = PBSession()
        if not isinstance(headers, dict):
            headers = {}
        try:
            data = http.get(url, headers=headers, timeout=self.urlTimeout).content
        except URLError:
            pBLogger.warning(self.errorRetrieve % self.name)
            return ""
        except HTTPError:
            pBLogger.warning(self.errorNotFound % url)
            return ""
        except (ssl.SSLError, socket.timeout):
            pBLogger.warning(self.errorTimedOut % self.name)
            return ""
        try:
            text = data.decode("utf-8")
        except Exception:
            pBLogger.warning(self.errorBadCodification % self.name)
            return ""
        http.close()
        return text

    def retrieveUrlFirst(self, search):
        """Retrieves the first bibtexs that the search gives,
        using the subclass specific instructions.

        Parameter:
            search: the string to be searched

        Output:
            returns None in the default implementation
                (must be subclassed)
        """
        return None

    def retrieveUrlAll(self, search):
        """Retrieves all the bibtexs that the search gives,
        using the subclass specific instructions

        Parameter:
            search: the string to be searched

        Output:
            returns None in the default implementation
                (must be subclassed)
        """
        return None

    def loadInterfaces(self):
        """Load the subclasses that will interface
        with the main websites to search bibtex info
        and saves them into a dictionary (`self.webSearch`).

        The subclasses are read scanning the package directory.
        """
        if self.loaded:
            return
        for method in self.interfaces:
            try:
                _temp = __import__(
                    "physbiblio.webimport." + method, globals(), locals(), ["WebSearch"]
                )
                self.webSearch[method] = getattr(_temp, "WebSearch")()
            except Exception:
                pBLogger.exception(self.errorImportMethod % method)
        self.loaded = True

    def retrieveUrlFirstFrom(self, search, method):
        """Calls the function retrieveUrlFirst
        given the subclass method.

        Parameters:
            search: the search string
            method: the key of the method in `self.webSearch`

        Output:
            None in the default implementation (must be subclassed)
        """
        try:
            return getattr(self.webSearch[method], retrieveUrlFirst)(search)
        except KeyError:
            pBLogger.warning(self.methodNotAvailable % method)
            return ""

    def retrieveUrlAllFrom(self, search, method):
        """Calls the function retrieveUrlAll given the subclass method.

        Parameters:
            search: the search string
            method: the key of the method in `self.webSearch`

        Output:
            None in the default implementation (must be subclassed)
        """
        try:
            return getattr(self.webSearch[method], retrieveUrlAll)(search)
        except KeyError:
            pBLogger.warning(self.methodNotAvailable % method)
            return ""


physBiblioWeb = WebInterf()
physBiblioWeb.loadInterfaces()



#!/usr/bin/env python
"""Test file for the physbiblio.argParser module.

This file is part of the physbiblio package.
"""
import sys
import traceback
import unittest
from io import StringIO
from unittest.mock import MagicMock, patch

from PySide6.QtWidgets import QApplication

try:
    from physbiblio import __version__, __version_date__
    from physbiblio.argParser import *
    from physbiblio.config import pbConfig
    from physbiblio.database import pBDB
    from physbiblio.export import pBExport
    from physbiblio.gui.mainWindow import MainWindow
    from physbiblio.gui.setuptests import *
    from physbiblio.setuptests import *
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())


@patch("logging.Logger.debug")
@patch("logging.Logger.info")
@patch("logging.Logger.warning")
@patch("logging.Logger.exception")
class TestParser(unittest.TestCase):
    """Tests for main parser and stuff"""

    @patch("sys.stdout", new_callable=StringIO)
    def assert_in_stdout_sysexit(self, function, expected_output, mock_stdout):
        """Catch and if test stdout of the function contains a string"""
        with self.assertRaises(SystemExit):
            function()
        string = mock_stdout.getvalue()
        for text in expected_output:
            self.assertIn(text, string)

    @patch("sys.stderr", new_callable=StringIO)
    def assert_in_stderr_sysexit(self, function, expected_output, mock_stderr):
        """Catch and if test stderr of the function contains a string"""
        with self.assertRaises(SystemExit):
            function()
        string = mock_stderr.getvalue()
        for text in expected_output:
            self.assertIn(text, string)

    def test_global(self, *args):
        """Test that the options are recognised correctly"""
        parser = setParser()
        for opt in ("-v", "--version"):
            with self.assertRaises(SystemExit):
                parser.parse_args([opt])
            self.assert_in_stdout_sysexit(
                lambda: parser.parse_args([opt]),
                ["PhysBiblio %s (%s)" % (__version__, __version_date__)],
            )
        for opt in ("-h", "--help"):
            with self.assertRaises(SystemExit):
                parser.parse_args([opt])
            self.assert_in_stdout_sysexit(
                lambda: parser.parse_args([opt]),
                [
                    "usage: PhysBiblio.exe [-h] [-p ",
                    "{citations,clean,cli,daily,dates,export,test,tex,update,weekly,gui}",
                ],
            )
        for opt in ("-p", "--profile"):
            profile = list(pbConfig.profiles.keys())[0]
            with patch("physbiblio.cli.cli", autospec=True) as mock, patch(
                "logging.Logger.info"
            ) as _i:
                parser.parse_args([opt, "%s" % profile, "cli"])
                self.assertIn(
                    "Starting with profile '%s', database" % profile, _i.call_args[0][0]
                )
                args = parser.parse_args([opt, "%s" % profile, "cli"])
                args.func(args)
                mock.assert_called_once_with()

    def test_subcommands_ok(self, *args):
        """Test that the options are recognised correctly"""
        parser = setParser()
        tests = [
            ["citations", call_citationCount],
            ["gui", call_gui],
            ["test", call_tests],
        ]
        for sub, func in tests:
            args = parser.parse_args([sub])
            self.assertIs(args.func, func)
        tests = [
            [
                "clean",
                "physbiblio.database.Entries.cleanBibtexs",
                [],
                ([pBDB.bibs], {"startFrom": 0}),
            ],
            [
                "clean",
                "physbiblio.database.Entries.cleanBibtexs",
                ["-s", "100"],
                ([pBDB.bibs], {"startFrom": 100}),
            ],
            [
                "clean",
                "physbiblio.database.Entries.cleanBibtexs",
                ["--startFrom", "1000"],
                ([pBDB.bibs], {"startFrom": 1000}),
            ],
            ["cli", "physbiblio.cli.cli", [], ([], {})],
            [
                "daily",
                "physbiblio.database.Entries.getDailyInfoFromOAI",
                [],
                (
                    [
                        pBDB.bibs,
                        (datetime.date.today() - datetime.timedelta(1)).strftime(
                            "%Y-%m-%d"
                        ),
                        datetime.date.today().strftime("%Y-%m-%d"),
                    ],
                    {},
                ),
            ],
            [
                "dates",
                "physbiblio.database.Entries.getDailyInfoFromOAI",
                ["2018-04-01", "2018-05-01"],
                ([pBDB.bibs, "2018-04-01", "2018-05-01"], {}),
            ],
            [
                "export",
                "physbiblio.export.PBExport.exportAll",
                ["testname"],
                ([pBExport, "testname"], {}),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": True,
                        "overwrite": False,
                        "removeUnused": False,
                        "reorder": False,
                        "updateExisting": False,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "-o"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": True,
                        "overwrite": True,
                        "removeUnused": False,
                        "reorder": False,
                        "updateExisting": False,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "--overwrite"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": True,
                        "overwrite": True,
                        "removeUnused": False,
                        "reorder": False,
                        "updateExisting": False,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "--ov"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": True,
                        "overwrite": True,
                        "removeUnused": False,
                        "reorder": False,
                        "updateExisting": False,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "-s"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": False,
                        "overwrite": False,
                        "removeUnused": False,
                        "reorder": False,
                        "updateExisting": False,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "--save"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": False,
                        "overwrite": False,
                        "removeUnused": False,
                        "reorder": False,
                        "updateExisting": False,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "--sa"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": False,
                        "overwrite": False,
                        "removeUnused": False,
                        "reorder": False,
                        "updateExisting": False,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "f3", "--sa"],
                (
                    [pBExport, ["f2", "f3"], "f1"],
                    {
                        "autosave": False,
                        "overwrite": False,
                        "removeUnused": False,
                        "reorder": False,
                        "updateExisting": False,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "-u"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": True,
                        "overwrite": False,
                        "removeUnused": False,
                        "reorder": False,
                        "updateExisting": True,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "--updateExisting"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": True,
                        "overwrite": False,
                        "removeUnused": False,
                        "reorder": False,
                        "updateExisting": True,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "-r"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": True,
                        "overwrite": False,
                        "removeUnused": True,
                        "reorder": False,
                        "updateExisting": False,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "--removeUnused"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": True,
                        "overwrite": False,
                        "removeUnused": True,
                        "reorder": False,
                        "updateExisting": False,
                    },
                ),
            ],
            [
                "tex",
                "physbiblio.export.PBExport.exportForTexFile",
                ["f1", "f2", "--reorder"],
                (
                    [pBExport, ["f2"], "f1"],
                    {
                        "autosave": True,
                        "overwrite": False,
                        "removeUnused": False,
                        "reorder": True,
                        "updateExisting": False,
                    },
                ),
            ],
            [
                "update",
                "physbiblio.database.Entries.searchOAIUpdates",
                [],
                ([pBDB.bibs], {"startFrom": 0, "force": False}),
            ],
            [
                "update",
                "physbiblio.database.Entries.searchOAIUpdates",
                ["-f"],
                ([pBDB.bibs], {"startFrom": 0, "force": True}),
            ],
            [
                "update",
                "physbiblio.database.Entries.searchOAIUpdates",
                ["--force"],
                ([pBDB.bibs], {"startFrom": 0, "force": True}),
            ],
            [
                "update",
                "physbiblio.database.Entries.searchOAIUpdates",
                ["-s", "100"],
                ([pBDB.bibs], {"startFrom": 100, "force": False}),
            ],
            [
                "update",
                "physbiblio.database.Entries.searchOAIUpdates",
                ["--startFrom", "100"],
                ([pBDB.bibs], {"startFrom": 100, "force": False}),
            ],
            [
                "weekly",
                "physbiblio.database.Entries.getDailyInfoFromOAI",
                [],
                (
                    [
                        pBDB.bibs,
                        (datetime.date.today() - datetime.timedelta(7)).strftime(
                            "%Y-%m-%d"
                        ),
                        datetime.date.today().strftime("%Y-%m-%d"),
                    ],
                    {},
                ),
            ],
        ]
        with patch(
            "physbiblio.database.PhysBiblioDB.commit", autospec=True
        ) as _comm_mock:
            for sub, func, options, expected in tests:
                with patch(func, autospec=True) as mock:
                    args = parser.parse_args([sub] + options)
                    args.func(args)
                    mock.assert_called_once_with(*expected[0], **expected[1])

    def test_subcommands_fail(self, *args):
        """Test that the options are recognised correctly."""
        parser = setParser()
        tests = [
            ["clean", ["-f"]],
            ["clean", ["-s"]],
            ["clean", ["-s", "abc"]],
            ["clean", ["--startfrom", "1000"]],
            ["cli", ["-t"]],
            ["daily", ["date"]],
            ["dates", ["date1"]],
            ["export", ["testname1", "testname2"]],
            ["gui", ["-p"]],
            ["test", ["-f"]],
            ["tex", ["f1"]],
            ["tex", ["f1", "f2", "--overWrite"]],
            ["tex", ["f1", "f2", "-save"]],
            ["update", ["-o"]],
            ["update", ["-s", "abc"]],
            ["update", ["--startFrom"]],
            ["weekly", ["any"]],
        ]
        with patch(
            "physbiblio.database.PhysBiblioDB.commit", autospec=True
        ) as _comm_mock:
            for sub, options in tests:
                with self.assertRaises(SystemExit):
                    args = parser.parse_args([sub] + options)

    def test_subcommand_tests(self, *args):
        """Test that the options are recognised correctly"""
        from physbiblio.setuptests import skipTestsSettings

        oldSettings = skipTestsSettings.copy()
        parser = setParser()
        tests = [
            [["test"], False, False, False, False],
            [["test", "-d"], True, False, False, False],
            [["test", "--database"], True, False, False, False],
            [["test", "-g"], False, True, False, False],
            [["test", "--gui"], False, True, False, False],
            [["test", "-l"], False, False, True, False],
            [["test", "--long"], False, False, True, False],
            [["test", "-o"], False, False, False, True],
            [["test", "--online"], False, False, False, True],
            [["test", "-d", "--online"], True, False, False, True],
        ]
        for options, _db, _gui, _lon, _onl in tests:
            skipTestsSettings.default()
            with patch("unittest.runner.TextTestRunner.run", autospec=True) as _run:
                args = parser.parse_args(options)
                call_tests(args)
                self.assertEqual(skipTestsSettings.db, _db)
                self.assertEqual(skipTestsSettings.gui, _gui)
                self.assertEqual(skipTestsSettings.long, _lon)
                self.assertEqual(skipTestsSettings.online, _onl)
                self.assertEqual(_run.call_count, 1)
        with self.assertRaises(SystemExit):
            args = parser.parse_args(["test", "-a"])
        skipTestsSettings = oldSettings

    def test_call_citationCount(self, *args):
        """test call_citationCount"""
        with patch(
            "physbiblio.database.Entries.getAll",
            return_value=[{"inspire": "a"}, {"inspire": "b"}],
        ) as _ga, patch("physbiblio.database.Entries.citationCount") as _cc, patch(
            "physbiblio.databaseCore.PhysBiblioDBCore.commit"
        ) as _c:
            call_citationCount("args")
            _ga.assert_called_once_with()
            _cc.assert_called_once_with(["a", "b"])
            _c.assert_called_once_with()

    def test_call_clean(self, *args):
        """test call_clean"""
        args = NameSpace()
        args.startFrom = 12
        with patch("physbiblio.database.Entries.cleanBibtexs") as _f, patch(
            "physbiblio.databaseCore.PhysBiblioDBCore.commit"
        ) as _c:
            call_clean(args)
            _f.assert_called_once_with(startFrom=12)
            _c.assert_called_once_with()

    def test_call_cli(self, *args):
        """test call_cli"""
        with patch("physbiblio.cli.cli") as _f:
            call_cli(args)
            _f.assert_called_once_with()

    def test_call_export(self, *args):
        """test call_export"""
        args = NameSpace()
        args.filename = "abc"
        with patch("physbiblio.export.PBExport.exportAll") as _f:
            call_export(args)
            _f.assert_called_once_with("abc")

    def test_call_tex(self, *args):
        """test call_tex"""
        args = NameSpace()
        args.texFiles = ["a", "b", "c"]
        args.bibFile = "bib"
        args.overwrite = "o"
        args.save = "s"
        args.updateExisting = "u"
        args.removeUnused = "r"
        args.reorder = "d"
        with patch("physbiblio.export.PBExport.exportForTexFile") as _f:
            call_tex(args)
            _f.assert_called_once_with(
                ["a", "b", "c"],
                "bib",
                overwrite="o",
                autosave="s",
                updateExisting="u",
                removeUnused="r",
                reorder="d",
            )

    def test_call_update(self, *args):
        """test call_update"""
        args = NameSpace()
        args.startFrom = 12
        args.force = "f"
        with patch("physbiblio.database.Entries.searchOAIUpdates") as _f, patch(
            "physbiblio.databaseCore.PhysBiblioDBCore.commit"
        ) as _c:
            call_update(args)
            _f.assert_called_once_with(startFrom=12, force="f")
            _c.assert_called_once_with()

    def test_cumulativeInspireDates(self, *args):
        """test cumulativeInspireDates"""
        with patch("physbiblio.database.Entries.getDailyInfoFromOAI") as _f, patch(
            "physbiblio.databaseCore.PhysBiblioDBCore.commit"
        ) as _c:
            cumulativeInspireDates("a", "b")
            _f.assert_called_once_with("a", "b")
            _c.assert_called_once_with()

    def test_call_dates(self, *args):
        """test call_dates"""
        args = NameSpace()
        args.start = "s"
        args.end = "e"
        with patch("physbiblio.argParser.cumulativeInspireDates") as _f:
            call_dates(args)
            _f.assert_called_once_with("s", "e")

    def test_call_daily(self, *args):
        """test call_daily"""
        with patch("physbiblio.argParser.cumulativeInspireDates") as _f:
            call_daily("args")
            _f.assert_called_once_with(
                (datetime.date.today() - datetime.timedelta(1)).strftime("%Y-%m-%d"),
                datetime.date.today().strftime("%Y-%m-%d"),
            )

    def test_call_weekly(self, *args):
        """test call_weekly"""
        with patch("physbiblio.argParser.cumulativeInspireDates") as _f:
            call_weekly("args")
            _f.assert_called_once_with(
                (datetime.date.today() - datetime.timedelta(7)).strftime("%Y-%m-%d"),
                datetime.date.today().strftime("%Y-%m-%d"),
            )

    def test_call_gui(self, *args):
        """test the call_gui function"""
        mw = MainWindow()
        mw.show = MagicMock()
        mw.raise_ = MagicMock()
        mw.recentChanges = MagicMock()
        mw.errormessage = MagicMock()
        with patch(
            "physbiblio.argParser.QApplication", return_value=globalQApp
        ) as _qa, patch(
            "physbiblio.gui.mainWindow.MainWindow", return_value=mw
        ) as _mw, patch(
            "physbiblio.config.ConfigurationDB.update"
        ) as _u, patch(
            "physbiblio.config.GlobalDB.commit"
        ) as _c, patch(
            "sys.exit"
        ) as _e:
            with patch.dict(
                pbConfig.params, {"openSinceLastUpdate": __version__}, clear=False
            ):
                call_gui([])
            _qa.assert_called_once_with(sys.argv)
            _mw.assert_called_once_with()
            mw.show.assert_called_once_with()
            mw.raise_.assert_called_once_with()
            _e.assert_called_once_with(0)
            self.assertEqual(mw.recentChanges.call_count, 0)
            with patch.dict(
                pbConfig.params,
                {"openSinceLastUpdate": __version__ + "abc"},
                clear=False,
            ):
                call_gui([])
            mw.recentChanges.assert_called_once_with()
            _u.assert_called_once_with("openSinceLastUpdate", __version__)
            _c.assert_called_once_with()

    @classmethod
    def tearDownModule(self):
        sys.excepthook = sys.__excepthook__


if __name__ == "__main__":
    unittest.main()
